/*   GSI2.c  - GSI_2 Standard Platform

  Refer to design spec for operation.
  base on single color version
   handles:
    RS-232  (Menu Prompt)
    KeyPad/LCD Operation   (lcd.c)
    X10 Interface Function (x10.c)
    DS1302 Serial Clock    (sclk.c) & (spi.c)
    DS1821 Temp Sensor     (temp.c)
    DS1822 Temp Sensor     (temp.c)
	IR Detector			   (ir.c)
	Atmel EEPROm		   (eeprom.c)

  RS232 and LCD/Keypad are foreground operation

  menu.c handles menuing on LCD/keypad/encoder
  rulez.c handles the event rules
  
  
    Version      	   Author   	 		Date           Comments
From Previous Platforms:
     1.00           hg.   12/5/97  Initial Development (CMM11)
     1.01	      hg    7/7/00	Updated base code (GSI1)
GSI-2 Platform:	 
    1.00         (www.gunnsys.com),hg.   2/8/02  Initial Development (GSI2A,GSI2B)
*/
#include <iogp32.h>
#include <stdio.h>
#include    <ctype.h>
#include    <string.h>   
#include    <stdlib.h>
#include <hc08def.h>
void init_devices(void);
int atoi(CONST char *);

/* Version Control */
#define MAJ_VER 1
#define MIN_VER 1

// Revision C PC board mods (light sense)
#define PCB_REVC 1 

#define TRUE 0xff
#define FALSE 0

// PWM Defines (see 68HC08GP32 datasheet)
#define TIM_STOP 0x20
#define TIM_RESET 0x10
#define TIM_UNBUF 0x10
#define TIM_TOV 0x02
#define TIM_COMPCLR 0xC
#define TIM_INTENAB 0x40
#define TIM_INTCLEAR 0x80


// PORT A (Outputs) Defines
#define PA_LCD_A0 0x02	  // LCD Address line
#define PA_LCD_CS 0x04    // LCD Chip Select (to LCD Controller)
#define PA_ONE_WIRE 0x08  // Dallas 1 wire Interface
#define PA_KP_MASK 0xF0	  // Mask for any keypadpress


// PORT B (Outputs&Inputs) Defines
#define PB_IO_MASK 0x3F  	 // mask for all the legit IO bits
#define PB_BLUE_MAN 0x20	 // Blue LED

#define PB_CLK_SCLK 0x40	 // RTC Clock SCLK line
#define PB_CLK_IO 0x80 	 	 // RTC IO data Line

#define PB_DEBUG_TOG 0x20    // port B bit 5 - use for scope trigger

// PORT C (Outputs) Defines
#define PC_SEL_EEPROM 1	   	 // Select EEPROM on SPI 
#define PC_SEL_LCD 2	 	 // Select LCD on SPI (latch toggle)
#define PC_SEL_KPAD 4 		 // select the keypad and IO (latch toggle) 
#define PC_SEL_EXT 0x10	 	 // select external SPI
#define PC_X10_IN 0x20	 	 // X-10 Recv. Line
#define PC_X10_OUT 0x40		 // X-10 TX Line
#define PC_X10_ZERO 0x08		 // X-10 Zero crossing Line

// PORT D (Outputs) Defines
#define PD_AUDIO_MASK 0x30 	 // Mask for audio bits
#define PD_IR_DETECTOR 0x40	 // IR Detector

// SPI Defines
#define SPI_CONFIG 0x22  /* ENABLED - for input/outputs enable, master, push-pull */
#define SPI_CONFIG2 0x0  /* ENABLED - for max Baud */

// GSI_2 Board Defines
// keypad / IO registers
#define KIO_RELAY 0x40	 // the 10AMP on board relays
#define KIO_RELAY2 0x20	 
#define KIO_BLUE 0x20	 // the blue LED
#define KIO_CLK_RST 0x80 // the DS1302 clock chip reset 
//' Reg 2
#define KIO_RED 0x1		 // RED health
#define KIO_GREEN 0x2	 // Green Health
#define KIO_LCDBACK 0x40  // LCD Backlight

// MAX size of eeprom (used to determine log roll-overs)
#define EEPROMSIZE 2047   // 511 for 25AT040, 127 for 25AT010 2047 for AT25160

/** PROTOTYPES **/
void init_hard(void);
void do_line(void);
int get_line(void);
unsigned char kbhit(void);
void delay(int q);
void caldelay(unsigned char delayval);
void buzz(unsigned char freq,unsigned int time);
void background(void);
void get_ad(void);
void run_pwm(unsigned int pwm_val);
void stop_pwm(void);
void relay_toggle(void);
void health_toggle(void);
void temp_probe_test(void);
void ad_test(void);
void help_print(void); 
void Set_Time(void);
void X10_Test(void);
unsigned int atox(char *tmp);
void Dump_Mem(void);
void eeprom_test(void);
void output_test(void);
void ir_test(void);
void do_status_display(unsigned char status);
void one_sec(void);
unsigned char read_light(void);
void set_relay(unsigned char relay, unsigned char state); 
void read_parms(char *stmp, char *stmp2);
void save_all(void);
void save_parms(void);
void save_event(unsigned char ev_num);
void default_parms(void);
void do_log(unsigned int event,unsigned char dtype);
void disp_log(void);
void log_reset(void);
void write_log_ptrs(void);
void set_tone(unsigned char xtone);
void check_tone(void); 
void config_setting(void);
void event_prog(void);
void do_status(unsigned char statustype, char *buf,unsigned char refresh);
void sched_prog(void);


#pragma interrupt_handler	timer_handler
#pragma interrupt_handler	ir_handler

unsigned int inc_val;		          /* incrementing counter used for timing */
volatile unsigned int delay_v = 0;	  /* wait routine working counter */
unsigned char  health=0;               /* one sec counter */
unsigned char healthcode=0;			  /* blink code */
unsigned char healthblink;			 // used to blink out health codes
unsigned int one_sec_pace;
unsigned char delaytimer; 			 // delay for secs
unsigned char delaytimerm; 			 // delay for mins


// SPI Port Ghosts
unsigned char kp_io_reg; 			 // key pad and i/o register ghost
unsigned char kp_io_reg2; 			 // key pad and i/o register ghost
unsigned char exp_io_reg;			 // axpansion IO register

// other toggles
unsigned char digitalout;				 // state of dighital outputs
unsigned char relay;				 // state of relays
unsigned char tone;				 // state of tone output

// Force toggles (bits)
unsigned char force=0;
#define FORCEHEALTH 1
#define FORCERELAY 2
#define FORCEOUTPUT 4
#define FORCETONE 8

// The Flag variable
unsigned char flag=0; 

unsigned char ledstate;				 // state of blue led

unsigned char lx_ir_fun;  // Last functions (used for status display)
unsigned char lx_x10_fun; //

// 
#define SLEEP(iy) delay_v=iy; while(delay_v);
/**** Varables ****/

unsigned char cmd[25];        /* serial port command buffer */

// EEPROM STUFF !!!

unsigned char main_x10_addr;		 // X10 Main Stuff
unsigned char main_x10_hc;

unsigned char tempunits;	 /* temp display units (0=Celsius) */

unsigned char relaymode;					 // relaymodeone Value
unsigned char logging;				 // logging type and flag (flag is MSB)
unsigned char logtype;				 // logging type

// Logging stuff
unsigned char logfirst;				 // start of log buffer EEPROM byte num
unsigned int lognext;				 // pointer to next EEPROM byte to log to
unsigned int logstart;				 // log start pointer (used to know where to roll
// log dtypes (WARNING 3 bit value - stored in hour)
#define LOG_GENERAL 1				 // Startup
#define LOG_EVENT 2					 // events
#define LOG_VALUE 3	 				 // value type start 
#define LOG_VALUE_TEMP 3	 		 // Temp A value type
#define LOG_VALUE_TEMPB 4			 // Temp Bvalue type 
#define LOG_VALUE_X10 5	 			 // X10 value type 
#define LOG_VALUE_LIGHT 6	 		 // Light value type
#define LOG_VALUE_ANLG 7			 // Analog type 

// Averaging stuff fro light sensor
#define LIGHTAVGMAXIX  7     // number of averages
unsigned char lightavg[LIGHTAVGMAXIX+1];	 //  to averaged values
unsigned char lightavgix;	 // index into avage array
unsigned int lightavgsum;	 // sum of averages

unsigned char numfired;				 // number of rules fired 
unsigned char blueflash;			 // (used to set speed  blue)


#define NUMEVENTS 16

// Local event buffer - used for copying events out of EEPROM
// 9 bytes each - 81 bytes total
// offset at byte 6 in eeprom 
struct event {
 	   unsigned char in1_type;  // type of input
 	   unsigned char in1_val[2];	// value or time, time - top two bits are minutes, 0,15,30,45)
	   unsigned char in_logic;	   // and/or/none 0 = none 1-7f AND event 81 -ff OR event
 	   unsigned char out1_type;  // type of output
 	   unsigned char out1_val[2];	// value
} ev = {2,{70,80},0,4,{3}};

#define NUMINPUTTYPES 12
#define INPUTTIME 1
#define INPUTTEMP 2
#define INPUTX10 3
#define INPUTLIGHT 4
#define INPUTIR 5
#define INPUTDIGITAL 6
#define INPUTANALOG 7
#define INPUTDOW 8
#define INPUTDLYS 9
#define INPUTDLYM 10
#define INPUTFLAG 11
#define INPUTTEMP2 12

const char input_names[NUMINPUTTYPES+1][5] = {
"...",
"TIME",
"TEMP",
"X10 ",
"LGHT",
"IR  ",
"INPT",
"ANLG",
"DAY ",
"DLYS",
"DLYM",
"FLAG",
"TMPB"
};

#define NUMOUTPUTTYPES 10
#define OUTPUTHEALTH 1
#define OUTPUTRELAY 2
#define OUTPUTOUTPUT 3
#define OUTPUTTONE 4
#define OUTPUTX10 5
#define OUTPUTDLYS 6
#define OUTPUTDLYM 7
#define OUTPUTFLAG 8
#define OUTPUTEXP 9

const char output_names[NUMOUTPUTTYPES][7] = {
"...",
"HEALTH",
"RELAY ",
"OUTPUT",
"TONE  ",
"X10   ",
"DLYS  ",
"DLYM  ",
"FLAG  ",
"EXPBRD"
};

const char dow_text[9][4] = {"SUN","MON","TUE","WED","THU","FRI","SAT","WKD","WKE"};

const char min_sel_text[4][3] = {"00","15","30","45"};

// END EEPROM STUFF



// junk - used all over the place - may be corrupted
volatile unsigned int i=0,q=0;
unsigned int memptr;


/**********************************************************/
/*********** LOCAL C FUNCTIONS (include instead of link!) **/
/**********************************************************/
#include "spi.c"    /* Serial Periph Intf (Bit bang) */
#include "temp.c"    /* for Temp Sense Clock Stuff */
#include "sclk.c"    /* for Serial Clock Stuff */
#include "eeprom.c"    /* for EEPROM */
#include "ir.c"		   /* IR Detector */
#include "x10.c"    /* for X-10 automation interface (TW-523) */
#include "lcd.c"    /* for LCD and keypad functions */
#include "menu.c"    /* for LCD and serial menu functions */
#include "rulez.c"    /* Rules (events)  Processing */


/******* INTERUPT HANDLERS ***********/

// IR handler
#ifdef IRLIB
void ir_handler() {
unsigned char xx;
    xx = T2SC0;
	do_ir();	   // get the function code or give up
	  
	ir_exit_delay=2;  // setup delay to turn back on the ints.
}
#endif

// TIMER_HANDLER  - interrupt handler about 3.3 ms at 9.8MHZ and max (32768) divider
// KEEP Quick and NO SPI 
void timer_handler() {

    if (delay_v)	  // delay counter goes down to zero for delays 
      delay_v--;
	
	  
    inc_val++;  // inc_value goes up to record timing
	
	// Handle the Status  (Blue) LED
		
    if ((health++ > blueflash) || (lcdmenumode == LCDMENUMODE_EVENTPGM)) { // status  check
	    health = 0;
        PTB = PTB ^ PB_BLUE_MAN;  // toggle blue status lamp for ok
    }
	
	if (dbounce) dbounce--;
	  
    if (one_sec_pace)
  	  one_sec_pace--;
	
	// handle turning on the IR ints
	if (ir_exit_delay) {
  	  ir_exit_delay--;
	  if (ir_exit_delay==0) 
	    T2SC0 = 0x4C;  // ok more ints now
	}
	TBCR |= 8;  // ACK Timer
}

	
/************** M A I N *************/
/* Start foreground - serial port handler */
void main()  {

	  char ch;
	  init_devices();
	  delaytimer=0;
	  disp_rule=0;
	  lightavgsum=0;
	  init_hard();
	  // Sign on display
	   /* startup display - */
      lcd_clr(); 
      sprintf(cmd,"Gunn Systems V%d.%2.2d\0", MAJ_VER,MIN_VER);
      printf("\n\r%s\n\r", cmd); 
      lcd_putl(cmd);
	  // bring up the temp sensors
      ds1822_num = temp_Init_1822();
      printf("\n\r%d  Temp sensors Found\n\r",ds1822_num);
#ifdef DS1821	
      if (ds1822_num == 99) {
       temp_Init_1821();  
       if (tempOK)  {
   	     printf("1 Temp sensor Found\n\r");
	   }
	  }
#endif	  
      Read_Temp();

	  // check to see if the menu key is being pressed at start-up (erase parms)
  	  nv = (PTA & 0xF0);  // get inputs bits
	  if ((!(nv & 0x10))) {
        lcd_clr(); 
	    lcd_putl("Press Enter to");
	    lcd_putl2("Load Defaults");
		delay_v=6000;
		while ((!(nv & 0x10)) && (delay_v)) // wait for release of menu key
    	  nv = (PTA & 0xF0);  // get inputs bits
		while ((nv & 0x10) && (nv & 0x20) && (delay_v)) 
   	      nv = (PTA & 0xF0);  // get inputs bits
		if (!(nv & 0x20)) { // load defaults  
 	      default_parms();
          read_parms(NULL,NULL);
		}
	  }
	  else
      // read in parms from eeprom
        read_parms(NULL,NULL);

 	  // init averages 
	    for (i=0;i<sizeof(lightavg);i++) 
 	      lightavg[i]=0;

      SLEEP(400);
      Read_Temp();
	  // reset min and max
 	  mintemp2=maxtemp2=curtemp2;
 	  mintemp=maxtemp=curtemp;

      do_status_display(0);
	  // check and enable X10
	  wait_trans();
	  X10ok=FALSE;
	  if (delay_v > 0) {
	    X10ok = TRUE;
	    printf("X10 Enabled\n\r");
	  }

      if (logging)  	 	// do logging - log startup
	    do_log(0xFFFE,LOG_GENERAL);		// FFFE = startup

      lcd_clr(); 
      lcd_MainMenu();
	  Display_Menu();

  	  relay=0;

 	  delay_v = 0;  // reset delays
  	  // Main loop forever
	  while (1) {
	     	printf("\n>> ");
      		get_line();  // Background called here 
      		do_line();	 // handle serial port command
	  }
}

// READ_PARMS - read parms from eeprom into RAM
void read_parms(char *stmp, char *stmp2)
{
unsigned char *xx;

  memptr = 0x100;  // high ram pointer
  q = eeprom_read(0);  // check signature
  if (q == 0x69) {  // signature that wee know we wrote
    tempunits = eeprom_read(1);  // tempunits read
    main_x10_addr = eeprom_read(2);  // addr read
    main_x10_hc = eeprom_read(3);  // hc  read
	relaymode = eeprom_read(4);
	logging = eeprom_read(5);
	logtype=logging;
    q=6;  // rest start at 6
	// Read events and save in high RAM
	for (prg_event=1;prg_event<=NUMEVENTS;prg_event++) { 
      xx = (unsigned char *) &ev;
      for (i=0;i<sizeof(ev);i++) {
	    *xx=eeprom_read(q++);
	    xx++;
	  }  
	  // copy it fro buffer into high mem
	  memcpy((void *)memptr,(void *)&ev, sizeof(ev));
	  memptr +=sizeof(ev);
	  if (((int)stmp != NULL) && (ev.in1_type > 0) && (ev.in1_type < NUMINPUTTYPES)) {
	    // print it out to the serial port 
        EventText(stmp,stmp2);
        printf("%s %s\n\r",stmp,stmp2);
	  }
	}
  }
  else { // bad signature
    printf("EEPROM  Invalid - %x\n\r",q);
    default_parms();
	q=(NUMEVENTS*sizeof(ev)) +6;
  } 
  
  // setup log pointers
  logfirst=q;
  logstart=eeprom_read(q++);
  logstart = logstart << 8;
  logstart +=eeprom_read(q++);
  lognext=eeprom_read(q++);
  lognext = lognext << 8;
  lognext +=eeprom_read(q++);
}

// DEFAULT_PARMS  - zero out
void default_parms(void)
{
unsigned char *xx;

    memptr = 0x100;  // high ram pointer
	// clear events and save in high RAM
	memset((void *)&ev,0,sizeof(ev));
	for (prg_event=0;prg_event<NUMEVENTS;prg_event++) { 
	  // copy it fro buffer into high mem
	  memcpy((void *)memptr,(void *)&ev, sizeof(ev));
	  memptr +=sizeof(ev);
    }	
	tempunits =1;
	relaymode=logging=logtype=0;
	main_x10_addr=15;
	save_all();
}

// SAVE_PARMS - save to EEPROm 
void save_parms(void)
{

   // write signature
   eeprom_write(0x69,0);
   eeprom_write(tempunits,1);
   eeprom_write(main_x10_addr,2);			
   eeprom_write(main_x10_hc,3);
   eeprom_write(relaymode,4);
   eeprom_write(logging,5);
}

// SAVE_ALL Stuff - save to EEPROm 
void save_all(void)
{
unsigned char z;

   lcd_clr();
   lcd_putl("Saving...");
   save_parms();  
   q=6;  // rest starts at 6
   memptr = 0x100;  // high ram pointer
   // Save events from high RAM
   for (z=0;z<NUMEVENTS;z++) {
 	  // copy it FROM high mem
	  memcpy((void *)&ev,(void *)memptr, sizeof(ev));
	  memptr +=sizeof(ev);
      save_event(z);
	}  
}

// SAVE_EVENT - save event info to EEPROm from ev structure 
void save_event(unsigned char ev_num)
{
unsigned char *xx;
      
     xx = (unsigned char *) &ev;
	 q = 6 + (ev_num * sizeof(ev));
     for (i=0;i<sizeof(ev);i++) {
	   // write data
       eeprom_write(*xx,q++);
	   if (eeprom_timeout) {
         printf("EEPROM Timeout!!\r");
	     return;
	   }
	   xx++;
	  }
}




/* DO_STATUS_DISLAY - Do current status and display to teh serial port */
void do_status_display(unsigned char status)
{
char thebuf[24];
  if (status == 0) {
    // do all status
    for (i=1;i<NUMINPUTTYPES;i++) {
      if ((i == INPUTDOW) || (i==INPUTDLYM))  // jump past delayM and date, day 
	    i++;
      thebuf[0] = 0;
	  do_status(i,thebuf,1);
	  if (thebuf[0]) 
	    printf("%s\n\r",thebuf);
    }
  }
  else {
    thebuf[0] = 0;
    do_status(status,thebuf,1);
    if (thebuf[0]) 
	    printf("%s\n\r",thebuf);
  }
  lx_x10_fun=lx_ir_fun=0;
}


/* DO_STATUS - format the text buffer with current data */
/* Status type is theinput types */
/* ir and x10 flags passed in */
/* refresh flag indicates to read input values (time, temp and light) */
void do_status(unsigned char statustype, char *buf,unsigned char refresh)
{
unsigned char a1,a2,a3,a4,a5;

        if ((statustype==INPUTTIME) || (statustype==INPUTDOW)) { // time or day
		  if (refresh)
	         Read_Time();	/* get current time values */
           sprintf(buf,"%2d:%02d.%02d %2d/%02d %s ",hour,min,sec, month,day,dow_text[dow-1]);
	    }
        else if (statustype==INPUTTEMP) {
            if ((ds1822_num) ) {
			  if (refresh) // in status display keep temp current
				#ifdef DS1821	
			        Read_Temp(); 
				#else
				    Read_Temp_1822();	/* number of ds1822 */
				#endif	

			  if ((ds1822_num==2) && (sec & 2))
			    a1='B',a2=conv_temp(curtemp2),a3=conv_temp_dp(curtemp2),a4=conv_temp(maxtemp2),a5=conv_temp(mintemp2);
			  else 
			    a1='A',a2=conv_temp(curtemp),a3=conv_temp_dp(curtemp),a4=conv_temp(maxtemp),a5=conv_temp(mintemp);
				
                sprintf(buf,"%c %2d.%d%c  Lo%2d Hi%2d",a1,a2,a3,(tempunits == 0)?'C':'F',a5,a4);
			}
			else
			  sprintf(buf,"No Temp Sensors");
	    }
        else if (statustype==INPUTX10) { // X10 Status
		  	if ((lx_x10_fun > 0) &&  (lx_x10_fun < 6)) {
			   a1 = 'A';
			   a1 += x10_hcode;
			   sprintf(buf,"X10 A:%c %d Funt: %s",a1,x10_addr,selectlist[0][lx_x10_fun]);
		    }
	    }
        else if (statustype==INPUTLIGHT) {  // Light sense
		    if (refresh) // in status display keep temp current
		      read_light();
			a1 =  lightavgsum >> 3;
            sprintf(buf,"Light %2d %%",a1);
	    }
        else if (statustype==INPUTIR) { // IR Detector
		  	if (lx_ir_fun > 0) {
			   sprintf(buf,"IR Fun: %d",lx_ir_fun);
		    }
	    }
        else if (statustype==INPUTANALOG) {  // Inputs
     	    ADSCR = 0x27;   // continious conversions	
 		    ADCLK=0x10;
   		    ADSCR = 0x20;   // continious conversions, select A/D
		  	while (!(ADSCR&0x80));
   		    a1 = ADR;
            sprintf(buf,"Analog>%d",a1);
	    }
        else if (statustype==INPUTDIGITAL) {  // Digital Inputs
            sprintf(buf,"Inputs>%d,%d,%d,%d",(PTB&0x01?1:0),(PTB&0x02?1:0),(PTB&0x04?1:0),(PTB&0x08?1:0));
	    }
        else if ((statustype==INPUTDLYS) || (statustype==INPUTDLYM)) { // Delays
 	        sprintf(buf,"DlyS %d, DlyM %d",delaytimer,delaytimerm);
	    }
        else if (statustype==INPUTFLAG) { // Flag
 	        sprintf(buf,"Flag=%d",flag);
	    }
        else if (statustype==INPUTTEMP2) {  // second temp sensor
            if ((ds1822_num) ) {
			  if (refresh) // in status display keep temp current
			    Read_Temp(); 
			  if (ds1822_num==2)
                sprintf(buf,"T2 %2d%c  Lo%2d Hi%2d",conv_temp(curtemp2),(tempunits == 0)?'C':'F',conv_temp(mintemp2),conv_temp(maxtemp2));
			}
	    }
}		


/* INIT_HARD - initialze Hardware */
void init_hard(void)
{

	  // SCI already turned on in _HC08Setup
	  // called by the startup code
	  SCBR = 0x2;  // 9600 baud
	  DDRA=0x06;   // only LCD_CS and LCD_A0 outputs
	  DDRC=0x57;   // Port C is our output (except X-10 recv.&Zero)
	  PTC=0;	   // 
	  PTD=0;
	  DDRD=PD_AUDIO_MASK+ 0x0F;   // turn on port D  outputs - timer channels
	  DDRB=0xE0;   // use PTB for I/O, RTC I/O start define as outputs.
	  TBCR=0x6;   // Start up 32768 divider timebase for timing 
	  PTAPUE=0xFF;	 // Enable pullups on port A (keypad buttons)
	  PTCPUE=0x28;
	  PTDPUE=0x02;
	  // SPI Init
	  SPCR=SPI_CONFIG;
	  SPSCR=SPI_CONFIG2;
	  T1SC= TIM_STOP | TIM_RESET;  // Stop Tim
   	  stop_pwm();  // no tone
	  T1SC = 0;		 // start counters for IR use
	  kp_io_reg=0x0C; // Select any of the keypad buttons
	  exp_io_reg=kp_io_reg2=0;
      spi_write2(kp_io_reg,kp_io_reg2,PC_SEL_KPAD);  // send it out
      spi_write(exp_io_reg,PC_SEL_EXT);  // send out expansion board
	  #ifdef ENCODER
    	  nv = (PTA & 0xF0);
	  #endif
  
      asm("cli"); // OK get interupts from here on
	  SLEEP(100);
      lcd_init(); 	 // get LCD display going 
	  eeprom_init(); // initialize the eeprom
	  init_ir();	 // init the IR counter channels
		
}

/* DO_LINE - handle ASCII type commands from serial */
void do_line(void)
{
unsigned char *ptr,val;

  	printf("\n\r");
  	switch(toupper(cmd[0])) 
  	{
   	/* Input Command */
    	case 'I':
			i = atox((char *)&cmd[2]);
	      	ptr = (unsigned char *)i;
	      	printf("Value at %X -> %X\n\r",i,*ptr);
	      	break;
    /* Output Command */
    	case 'O':
	      	ptr = (unsigned char *)atox((char *)&cmd[2]);
	      	i=2;
	      	while (cmd[i] != ' ')
	        	i++;
	      	i++;
	      	val = atox((char *)&cmd[i]);
	      	*ptr=val;
	      	printf("Value set %X = %X\n\r",ptr,val);
	      	break;
    /* Dump Command */
    	case 'D':
			Dump_Mem();
	      	break;
   	/* Sound Command */
    	case 'Q':
			i = atoi((char *)&cmd[2]);
			force |= FORCETONE;
			set_tone(i);
	      	printf("Tone %d\n\r",i);
		      //buzz(i,100);
	      	break;
   	/* Relay Toggle */
    	case 'R':
		    relay_toggle();
	      	break;
   	/* Health Toggle */
    	case 'H':
		    health_toggle();
	      	break;
	/* EEPROM Utilities */
 		case 'E':
		  eeprom_test();
	    	break;
	/* Config Settings */
 		case 'C':
		  config_setting();
	    	break;
	/* Default settings */
 		case 'A':   // deafult Parms
		   printf("Really Load Defaults (Y/N) ?");
   	       cmd[0] = getchar();
  	       putchar(cmd[0]);	  // Echo back duplex
		   if (toupper(cmd[0]) == 'Y') {
   	         default_parms();
		   }
  	       printf("\r\n");
		   break;
	/* Clear Forces */
 		case 'F':
           force=0;
		  printf("Forces Cleared\r\n");
	    	break;
	/* Digital OUTPUTS */	
 		case 'J':
		  output_test();
	      break;
	/* Display Log */
 		case 'L':
		  disp_log();
	      break;
	/* Event Programming */
 		case 'P':
		  event_prog();
     	  break;
    /* Set Time and date */
   	    case 'S':
	  	    Set_Time();
	  	 	break;
	/* X-10 Test */
      	case 'X':
		      X10_Test();
      		  break;
			  
    /* Status Command */
    	case 'Z':
		    if ((cmd[2] > '0') && (cmd[2] < '9'))
  			  do_status_display(cmd[2]-'0');
			else
  			  do_status_display(0);
	      	break;
    case '?':
	        help_print();
			break;
      default:
      		printf("\nInvalid Command, Use ? for help - \n\r");
  	}
    cmd[2]=0;  // clear out, optional flag values
    printf("\n\r");
}

/* HELP_PRINT  - show help when ? entered */
void help_print() 
{
			printf("Available Commands:\n\r");
			printf("I <adrs> - Read Memory & I/O (Hex Entry)\n\r");
			printf("O <adrs> <val> - Set Memory & I/O Values (Hex Entry)\n\r");
			printf("D <adrs> <number> - Dump Memory (Hex Entry)\n\r");
      		printf("S <hour> <min> <month> <day> <year> - Set Time/Date\n\r");
			printf("E [RWD] <adrs> <val> - EEPROM Read/Dump/Write\n\r");
			printf("Q <value> - Set Tone (0-9)\n\r");
			printf("H <val> - Set Health (0-4)\n\r");
			printf("J <num> <val>- Actvate Outputs\n\r");
			printf("R <num> <val> - Relay Activate\n\r");
			printf("C <T,R,X,H,L,?> <val> - Configure Settings\n\r");
			printf("L - Display Log\n\r");
			printf("Z <0-9>- Status and Display Inputs\n\r");
			printf("F - Clear Forces\n\r");
			printf("P <?,1-9> - Event Program\n\r");
			printf("A - Load Default Parameters\n\r");
			printf("X - X-10 Test\n\r");
}


/* DUMP_MEM */
void Dump_Mem(void)
{
unsigned char *ptr,val;

	      	ptr = (unsigned char *)atox((char *)&cmd[2]);
	      	i=2;
	      	while (cmd[i] != ' ')
	        	i++;
	      	i++;
	      	val = atox((char *)&cmd[i]);
	      	//*ptr=val;
			while (val > 0) {
	      	  printf("\n\r%X ->",ptr);
			  for (i=0;i<16;i++) {
	      	    printf("%0X ",*ptr);
				ptr++;
			  }
			  if (val > 16) val -= 16;
			  else val=0;
			}  
}



/* RELAY_TOGGLE */
void relay_toggle()
{

     force|=FORCERELAY;
  	 if (cmd[2] == '2') {
  		    kp_io_reg &= ~KIO_RELAY2;
		    if (cmd[4] == '1') 
			  kp_io_reg |= KIO_RELAY;
			else 
			  kp_io_reg &= ~KIO_RELAY;
  	}
 	 else {
 		   kp_io_reg &= ~KIO_RELAY;
		    if (cmd[4] == '1') 
			  kp_io_reg |= KIO_RELAY2;
			else 
			  kp_io_reg &= ~KIO_RELAY2;
  	}			
	spi_write2(kp_io_reg,kp_io_reg2,PC_SEL_KPAD);  // send it out
	printf("Relay Forced");
}

/* OUTPUT TEST - walk the outputs */
void output_test(void)
{
unsigned char outp;
unsigned char val;

    outp = atoi((char *)&cmd[2]);
    val = atoi((char *)&cmd[4]);
    force |= FORCEOUTPUT;
	if (outp < 5) {
  	  digitalout=2 << (outp);
      if (val) {  // ON
	    kp_io_reg2 |= digitalout;
	  }
      else { 		   // OFF 
	     kp_io_reg2 &= ~digitalout;
	   }
	 }
	 else {
  	  digitalout=0x80 >> (outp-5);
      if (val) {  // ON
	    exp_io_reg |= digitalout;
	  }
      else { 		   // OFF 
	     exp_io_reg &= ~digitalout;
	   }
	 }
	  
    printf("Ouput Forced");
}



/* DISP_LOG - display logging */
void disp_log(void)
{
unsigned char zz;
char zh;
char stmp1[30];

 
  lcdentryvalue=0;
  zz = 0; 
  while (zz == 0) {
    zz = Display_log(cmd,stmp1);
    printf("%-20.20s ",cmd);
    printf("%-20.20s\r\n",stmp1);
	lcdentryvalue++;
  }
  printf("Clear Log?");
  inc_val=1;
  while (inc_val && !kbhit());
  if (kbhit()) {
    zh = getchar();
    putchar(zh);	  // Echo back duplex
    if (toupper(zh) == 'Y') {
     log_reset();
    }
  }
}




/* LOG_RESET - tryt to reset log */
void log_reset(void)
 {
			   
   // write FF to indicate end of log
    printf("\n\rLogging Reset\n\r");
    logstart=0;
	lognext=logfirst+4;  // reset pointers
	write_log_ptrs();
}


/* LED_TOGGLE - LED Lamp test */
void health_toggle()
{

   	healthcode = atox((char *)&cmd[2]);
	force|=FORCEHEALTH;
   	printf("Health Forced %d",healthcode);
}


/* SET_TIME */
void Set_Time(void)
{
#ifdef SCLKLIB
unsigned char col;
unsigned char val;

   	  	 i=2;
         sclk_Write(SCLK_SEC,0);
      	  for (col=0;col<6;col++) {
           switch (col) {
             case 0:   /* hour */
		 	   val = atoi((char *)&cmd[i]);
               sclk_Write(SCLK_HOUR,to_BCD(val));
               break;
             case 1:  /* Minute */
		 	 val = atoi((char *)&cmd[i]);
               sclk_Write(SCLK_MIN,to_BCD(val ));
               break;
             case 2:  /* Month */
		 	 val = atoi((char *)&cmd[i]);
               sclk_Write(SCLK_MONTH,to_BCD(val));
               break;
             case 3:  /* Day */
		 	 val = atoi((char *)&cmd[i]);
               sclk_Write(SCLK_DAY,to_BCD(val));
               break;
             case 4:  /* Year */
		 	 val = atoi((char *)&cmd[i]);
               sclk_Write(SCLK_YEAR,to_BCD(val));
               break;
             case 5:  /* Dow */
		 	   val = atoi((char *)&cmd[i]);
               sclk_Write(SCLK_DOW,to_BCD(val));
               break;
           }
           /* Parse to next entry */
   	     while ((cmd[i] != ' ') && (cmd[i] != '\0'))
          	i++;
           if (cmd[i] == '\0') break;
           i++;
           if (cmd[i] == '\0') break;
  	    
         }
         sclk_Write(SCLK_SEC,0);
#endif
}

/* X10_Test */
void X10_Test(void)
{
#ifdef X10LIB	
      		   printf("Line Crossing (60hz)");
	   		  wait_trans();
			  if (delay_v==0) {
      		    printf("FAILED!!\n\r");
			  }
	   		  inc_val=0;
	   		  wait_trans();
	   		  wait_trans();
	   		  wait_trans();
	   		  wait_trans();
	   		  i = inc_val;
			  #ifdef DEBUGX10
      		  printf(" -> %d ms\n\r", ((i * 33) / 40));
      		  printf("Waiting on command\n\r");
			  #endif
     		  printf("Got->%X\n\r",get_X10());
      		  printf("CX->%X - %X\n\r",cx1,cx2);
#endif
}


/* EEPROM_TEST */
void eeprom_test()
{
unsigned int addr,val;

 	 PTC |= PC_SEL_EEPROM;
	 delay(2);
	 i=0;
     q = atox((char *)&cmd[4]);
     if (toupper(cmd[2]) == 'W') {
      	   i=4;
	       while (cmd[i] != ' ') i++;
	       i++;
	       val = atox((char *)&cmd[i]);
 		   eeprom_write(val,q);
	 }
     if (toupper(cmd[2]) == 'D') {
	       printf("logfirst %X\n\r",logfirst);
		   for (q=q;q<EEPROMSIZE;q++) {
			 if (!(q & 0xF))   // every 16 bytes new line
			   printf("\n\r %X>",q);
  		     i = eeprom_read(q);
             printf("%0X ",i);
		   }
	 }
     else { 
		   i = eeprom_read(q);
           printf("EEPROM at %X = %X\r",q,i);
	 }
}


// CONFIG_SETTING - configure settings 
void config_setting(void)
{
unsigned int val;
char chx;
     if (toupper(cmd[2]) == '?') { // Status the Mode
 	  printf("Relay Mode: %s\n\r",selectlist[3][relaymode]);
 	  printf("Logging:    %s\n\r",selectlist[5][logging]);
 	  printf("Temp Units: %s\n\r",selectlist[2][tempunits]);
	  if (main_x10_hc < 16) {
	    chx = 'A';
	    chx += main_x10_hc;
  	    printf("House Code: %c\n\r",chx);
	  }
	  else
  	    printf("House Code: OFF\n\r");
      printf("X10 Addr:%d\n\r",main_x10_addr+ 1);
	  
	  return;
	 }
	 val = atoi((char *)&cmd[4]);
	 if (toupper(cmd[4]) == 'O') {
       val =1;
  	   if (toupper(cmd[5]) == 'N')  // ON 
	     val =1;
	   else if (toupper(cmd[5]) == 'F')  // ON	 	// OFF
	     val =0;
	 }
	 else if (toupper(cmd[4]) == 'F')  // Fah 
	     val =1;
	 else if (toupper(cmd[4]) == 'E')  // Events 
	     val =1;
	 else if (toupper(cmd[4]) == 'V')  // value 
	     val =2;
	 else if (toupper(cmd[4]) == 'A')  // All 
	     val =3;
	 else if (toupper(cmd[4]) == 'N')  // Fah 
	     val =0;
     if (toupper(cmd[2]) == 'T')  // Temp Mode
 	  tempunits =val;
     else if (toupper(cmd[2]) == 'R')   // Relay Mode
  	  relaymode =val;
     else if (toupper(cmd[2]) == 'H')   // House Code
		main_x10_hc =(toupper(cmd[4]) - 'A');
     else if (toupper(cmd[2]) == 'X')   // X10 Code
		main_x10_addr =val-1;
     else if (toupper(cmd[2]) == 'L') {   // Logging on/off
		logging=val;
		if (logging != logtype) {  // just changed - clear log
			   log_reset();
			   logtype=logging;
		}
	 }
     save_parms();

}
// EVENT_PROG - Program Events 
void event_prog(void)
{
char stmp[30];
char stmp2[25];

     if (toupper(cmd[2]) == '?') { // Status the events - read them back in
	 	read_parms(stmp, stmp2);
		printf("\n\r");
		return;
     }
	 if (cmd[2] == '\0') cmd[2] = '1',cmd[3]='\0';  // default if nothing
     if ((cmd[2] <= '9') && (cmd[2] >= '1')) { // program
 	      prg_event=atoi(&cmd[2]);  // get event to program
          lcdmenumode=LCDMENUMODE_EVENTPGM;
		  prg_seq=0;
	      lcdselectlist=0;
		  inc_val=1;
		  // COPY it into the local editing buffer
		  memptr = 0x100 + ((prg_event-1) * sizeof(ev));  // high ram pointer
   		  // copy it FROM high mem
 	 	   memcpy((void *)&ev,(void *)memptr, sizeof(ev));
		   curloc=0;
		   printf("Editing %d, ENTER=Next, X=Exit, U/D/0-9=Select\r\n",prg_event);
		   while ((lcdmenumode==LCDMENUMODE_EVENTPGM) && inc_val) {
		     // Generate and display the text buffer for event  
		     EventText(stmp,stmp2);
	  	     printf("\r%-20.20s %-20.20s\r",stmp,stmp2);
			 if (curloc > 20) curloc -=(0x40-20); // fix for LCD size vs term line 
		     for (i=0;i<curloc;i++) {
			     putchar(0x1B);  // ANSI FORWARD ARROW
			     putchar(0x5B);
			     putchar(0x43);
			 }
  	         while (!kbhit());
    	     cmd[0] = getchar();
			 putchar(cmd[0]);
			 if (cmd[0] == 0x1B) {  // COntrol Char comming in
    	       cmd[0] = getchar();
    	       cmd[0] = getchar();
			   if (cmd[0] == 0x41) cmd[0] = 'U';  // ANSI UP
			   if (cmd[0] == 0x42) cmd[0] = 'D';  // ANSI DOWN
			   if (cmd[0] == 0x44) cmd[0] = 'D';  // ANSI LEFT
			   if (cmd[0] == 0x43) cmd[0] = 'U';  // ANSI RIGHT
			 }
			 if (cmd[0] == '\r') cmd[0] = ENTER;
			 if (cmd[0] == 'X') cmd[0] = EXIT;
			 //if (cmd[0] < '0') printf(".%X",cmd[0]);
			 // handle the event programming entry
			 handle_eventpgmpress(cmd[0]);
		   }
		   if (cmd[0] != 'C')
		     printf("\r\nEvent Entered");
	 }
}


/* kbhit() - 1 = YES char waiting */
unsigned char kbhit()
{
  return(SCS1&0x20?1:0);
}

/* GET_LINE - get a line of ASCII text */
// process background while waiting
int get_line()
{
int cptr=0;

  	do 
  	{
	    while (!kbhit())
		  background();
    	cmd[cptr] = getchar();
		putchar(cmd[cptr]);	  // Echo back duplex
    	if (cmd[cptr] == '\b')    /* handle backspace */
    	{ 
      		cptr -= 2;
      		printf(" \b");
    	}
  	} while(cmd[cptr++] != '\r');
  	cmd[cptr]='\0';
  	return(cptr);
}

// BACKGROUND - background processing
// non-premptive -  MUST return after processing
void background(void)
{
#ifdef KEYPAD
// Handle the keypad in background
     user_input_data=read_keypad(); 	/* scan the key pad */
    /* background front panel ops */
     if (keypad_press()) {
	   do_keypad();
		// any keypad press turn off our status
  	    blue = 0;
		kp_io_reg &= ~KIO_BLUE;
		healthcode=0;
		kp_io_reg2 |= KIO_LCDBACK;
	 }
#endif
#ifdef ENCODER
// handle the encoder in the background
  user_input_data=encoder_in();
  if (user_input_data > 0) {
	kp_io_reg2 |= KIO_LCDBACK;
    if (lcdmenumode==LCDMENUMODE_RUNSTATUS) {
	  lcd_MainMenu();   /* go to main menu display after timeout*/
	  Display_Menu();
	}
	else
      handle_keypress(user_input_data);
  }
#endif

	 // Background calls the once per sec when it rolls around (safe for SPI)
     if (one_sec_pace == 0) {
       one_sec_pace = 303;  // just approx 1 sec
       one_sec();   
     }
	 
#ifdef X10LIB	 
 if (X10ok == TRUE) { 	   /* handle X10 trans/recv in background */
       background_x10(); 
  }
#endif	


}

// CHECK_TONE - handle tone at one sec
void check_tone(void) 
{
  if (tone < 5)   // turn off tone
    stop_pwm();
  if (tone==2) { 	// once per minute tone
     if (sec == 0)
      run_pwm(5000);
  }
  else if (tone==3) {	   // once per 8 sec tone
    if((sec & 7) == 0) 
       run_pwm(5000);
  }
  else if (tone==4) {  // once per sec tone
     run_pwm(4000);
  }
  else if (tone==9) {  // alternating tone
    if (sec & 1) 
     run_pwm(900);
	else
     run_pwm(1200);
  } 

}

/* ONE_SeC - handle once a second stuff */
/* Called from background after health counter reaches 0 in RTI */
void one_sec(void)
{
int i,xp;
unsigned int q;

  if (delaytimer) delaytimer--;
  sec++;
  if (sec > 59) {   // Once a minute processing
       Read_Time();	/* get current values, light, temp, etc */
       Read_Temp(); 
       read_light();
       if (delaytimerm) delaytimerm--;  // handle delay timer minute
	   // once a minute
	   if (min == 0) {
	     if (logging > 1) logging |=0x80;  // set the log now  flag to do logging in rulez
	   } 
  }

  check_tone();
  /* timeout to status display */
  if (lcdmenutimeout) {
    lcdmenutimeout--;
    if (lcdmenutimeout==0) {
	   lcdmenumode=LCDMENUMODE_RUNSTATUS;
	   // clear for new status
	   lcd_clr();
	   // reset lcd backlight
  	   kp_io_reg2 &= ~KIO_LCDBACK;
    }
  }
	// Handle the Status LED	
  if (healthcode) {   // we have a code to blink out
     kp_io_reg2 &= ~KIO_GREEN;
	  if (healthblink < (healthcode*2))
 	     kp_io_reg2 ^= KIO_RED;  // turn off for rest of cycle
	  else 
 		 kp_io_reg2 &= ~KIO_RED; // toggle status lamp for ok
	  healthblink++;
	  if (healthblink > 8)  // up to 4 codes I think
	    healthblink=0;
	}
	else {
		 kp_io_reg2 &= ~KIO_RED; // toggle status lamp for ok
	     kp_io_reg2 |= KIO_GREEN;
  }
	// REfreash SPI registers
  spi_write2(kp_io_reg,kp_io_reg2,PC_SEL_KPAD);  // send it out
  spi_write(exp_io_reg,PC_SEL_EXT);  // send out expansion board
	
  // call any display tasks
  background_lcd();
	
 
  rule_process();
}

void set_tone(unsigned char xtone)
{

  
  if (xtone == 0) {
    tone=0;
    stop_pwm();
    return;
  }
  if (tone != xtone) {
    tone = xtone;
    if (tone < 5)   
      run_pwm(5000);
    else {
      run_pwm(10000 - (tone * 1000));
    }
  }
}

// RUN_PWN - send out pulses out PD4 
void run_pwm(unsigned int pwm_val)
{
	  
	  // set up timer 1
	  T1SC= TIM_STOP | TIM_RESET;  // Stop Tim
	  T1MODH = pwm_val >> 8;
	  T1MODL = pwm_val & 0xFF;
	  T1CH0H = pwm_val >> 9;		   // 1/2 of pwm
	  T1CH0L = (pwm_val >> 1) & 0xFF;
	  T1SC0 = TIM_UNBUF | TIM_TOV | TIM_COMPCLR; // Start Channel

	  T1SC= 0;  // start timer

}


// STOP_PWM - Stop the encoder pulses
void stop_pwm(void)
{
	  
	  T1SC= TIM_STOP | TIM_RESET;  // Stop Tim 1
}



/* BUZZ - buzz the speaker*/
void buzz(unsigned char freq,unsigned int time) {
volatile unsigned int q;

 for (q=0;q<time;q++) {
   caldelay(freq);
   PTD = (PTD & 0x0F) | (hcode[q&0x0f] << 4);
 }
}

/* timer DELAY -  aprox 3ms per count*/
void delay(int q)
{
  delay_v=q;
  while(delay_v);
}

/* caldelay - calibrated loop delay function for quick delays */
/* Approx  12.5 uS delay per */
void caldelay(unsigned char delayval)
{
unsigned char i;
    for (i=0;i<delayval;i++) i=i;
}

/* READ_LIGHT - Read Light Sensor and average */
/* Return aprox 0-100 */
unsigned char read_light(void)
{
unsigned int perx,per;
unsigned char *ptx;  // far pointer

    ADSCR = 0x27;   // continious conversions	
    ADCLK=0x10;
#ifdef PCB_REVC
    ADSCR = 0x24;   // REV C uses Port B bit 4 continious conversions, select A/D
#else
    ADSCR = 0x23;   // continious conversions, select A/D
#endif	
	while (!(ADSCR&0x80));
   	per = 255 -  (unsigned char)ADR;
	per = per * 100;
	perx = per >> 8;
	lightavgsum -= lightavg[lightavgix];  // move out the old
	lightavg[lightavgix] = perx;   		  // sav ethe new
	lightavgix++;
	if (lightavgix > LIGHTAVGMAXIX)  // pointer to next data in averaging
  	  lightavgix=0;
	lightavgsum += perx;  // keep up counter
	per = lightavgsum >> 3;
	return((unsigned char) per);
}

/* SET_RELAY - Sets the relay true or false */
void set_relay(unsigned char relay, unsigned char state) 
{
  if (state) {  // ON
    if (relay) kp_io_reg |= KIO_RELAY;
    else kp_io_reg |= KIO_RELAY2;
  }
  else { 		   // OFF RELAY
    if (relay) kp_io_reg &= ~KIO_RELAY;
    else kp_io_reg &= ~KIO_RELAY2;
  }
}

/* DO_LOG - handle log entry */
void do_log(unsigned int event,unsigned char dtype)
{
  if (!eeprom_timeout) {  // EEPROM OK
	PTC |= PC_SEL_EEPROM;
	delay(2);
  // Write EVENT Number 
    eeprom_write((unsigned char)event,lognext);
    lognext++;
    eeprom_write((unsigned char)(event >> 8),lognext);
    lognext++; 

   // Write log time
    eeprom_write((hour +(dtype<<5)),lognext);  // hour entry has data tytpe as upper nibble
    lognext++;
    eeprom_write(min,lognext);
    lognext++;
  // write Date   
    eeprom_write(month,lognext);
    lognext++;
    eeprom_write(day,lognext);
    lognext++;
    // Bump log pointer
    if (lognext+7 > EEPROMSIZE) { // logging limit?? roll over 
     lognext=logfirst+4;
 	 logstart=logfirst+16;
    }
    else {
     if (logstart) {
       logstart+=6;
 	   if (logstart+7 > EEPROMSIZE) logstart=logfirst+4;
	 }
    }
    write_log_ptrs();
   //printf("Logged %d at %d:%d - nxt%d strt%d\n\r",event+1,hour,min,lognext,logstart);
  }
}

/* WRITE_LOG_PTRS - write the pointers to EEPROM at beginning of log */
void write_log_ptrs(void)
{
unsigned char chc;

   chc = (logstart >> 8);
   eeprom_write(chc,logfirst);
   eeprom_write(logstart,logfirst+1);
   chc = (lognext >> 8);
   eeprom_write(chc,logfirst+2);
   eeprom_write(lognext,logfirst+3);
   
   // write FF to indicate end of log
   eeprom_write(0xFF,lognext);
   eeprom_write(0xFF,lognext+1);
}
 
/* ATOX - ASCII to HEX */
unsigned int atox(char *tmp)
{
unsigned int ii=0;
  while ((*tmp > 0x20) && (*tmp != ' ')) {
    *tmp = toupper(*tmp);
    if ((*tmp < '0') || (*tmp > 'F')) {  // BAG - Invalid
	  return(ii);
	}
	if (*tmp > '9')
	  ii += *tmp-'A' + 0x0A;
	else
	  ii += *tmp-'0';
	tmp++;
	if ((*tmp > 0x20) && (*tmp != ' '))
	  ii = ii << 4;  // another digit comming
  }
  return(ii);
}

//call this routine to initialise all peripherals
void init_devices(void)
{
 //stop errant interrupts until set up
 CLI(); //disable all interrupts
	SCC1 = 0x40;	/* enable SCI */
	SCC2 = 0x0C;	/* enable both TX and RX */
	CONFIG1 = 0x01;	/* disable COP */
	CONFIG2 = 0x01;	/* Internal clock for SCI */
    COPCTL = 0x00;

 SEI(); //re-enable interrupts
}

