/*   eeeprom.c  - Serial EEPROM  (AT25010) handling

  handles:
    ATmel serial type EEPROMS (AT2010, AT25040)

Notes: These routines read and write the eeprom. Them DO NOT check for limits or verify data 	
	
	MAJOR ROUTINES:
void eeprom_init()- initialize the chip (enable writes)
void eeprom_write(unsigned char dat, unsigned int add) - write 1 byte
unsigned char eeprom_read(unsigned int add) - read 1 byte
unsigned char eeprom_status(void)  - read status 1 byte


    Version      Author   Date           Comments
     1.00           hg.   3/13/02  Initial Development
*/
/** PROTOTYPES **/
void eeprom_init(void);

#define EEPROMLIB 1
//#define EEPROMDEBUG  1  		   // Uncomment for debugging output
#define AT160		   // for AT25160 EEPROM, remove for AT25010 or AT25040

// EEPROM COmmands
#define EEPROM_WRITEENABLE 6
#define EEPROM_WRITE 2
#define EEPROM_READ 3
#define EEPROM_READSTATUS 5
#define EEPROM_WRITESTATUS 1

// Prototypes
void eeprom_init(void);
void eeprom_write(unsigned char dat, unsigned int add);
unsigned char eeprom_read(unsigned int add);
unsigned char eeprom_status(void);

unsigned char eeprom_timeout;

// EEPROM_INIT - initialize the chip (enable writes)
void eeprom_init()
{
     
 	 PTC |= PC_SEL_EEPROM;
	 caldelay(3); 
 	 PTC &= ~PC_SEL_EEPROM;
	 spi_write(EEPROM_WRITEENABLE,0 );	   	// write enable inst.
	 caldelay(3); 
 	 PTC |= PC_SEL_EEPROM;
	 eeprom_timeout=0;
	  
}

// write 1 byte
void eeprom_write(unsigned char dat, unsigned int add)
{
unsigned char xx;


    eeprom_init();  // enable write 
    delay_v=10;
     while ((eeprom_status() == 0xFF) && (delay_v));
	 if (!delay_v) {  // timeout error
  	   eeprom_timeout=1;
	   return;
	 }
  	 PTC &= ~PC_SEL_EEPROM;
	 caldelay(3); 
#ifndef AT160
	 if (add > 0xFF) {
	   add = add - 0x100;
  	   spi_write(EEPROM_WRITE | 0x08,0 ); // bit 8 address bit in command
	 }
	 else
#endif	 
  	   spi_write(EEPROM_WRITE,0 );
#ifdef AT160
	 spi_write(add>>8,0 );
#endif	   
	 spi_write(add,0 );
	 spi_write(dat,0 );
	 caldelay(3); 
  	 PTC |= PC_SEL_EEPROM;
#ifdef EEPROMDEBUG
	 printf("WRT-EE %X,%X..",dat,add);
#endif	
   delay(10);
}


// read 1 byte
unsigned char eeprom_read(unsigned int add)
{
unsigned char xx;

  	 PTC &= ~PC_SEL_EEPROM;
#ifndef AT160
	 if (add > 0xFF) {
	   add = add - 0x100;
  	   spi_write(EEPROM_READ | 0x08,0 ); // bit 8 address bit in command
	 }
	 else
#endif
	   spi_write(EEPROM_READ,0 );
#ifdef AT160
	 spi_write(add>>8,0 );
#endif	   
	 spi_write(add,0 );
	 caldelay(3); 
	 xx = spi_read();
  	 PTC |= PC_SEL_EEPROM;
	 return(xx);
}

// read status 1 byte
unsigned char eeprom_status(void)
{
unsigned char xx;

 	 PTC &= ~PC_SEL_EEPROM;
	 caldelay(3); 
	 spi_write(EEPROM_READSTATUS,0 );
	 caldelay(3); 
	 xx = spi_read();
  	 PTC |= PC_SEL_EEPROM;
	 return(xx);
}
