/*   ir.c  - Infa-red Detector handler

 Uses  Timer 2 channel 0

Note: The IR detectors is a PNA4602 and detects 38Khz IR.
Use check_ir to check for any activity (put in background processing loop or int). 
The check_ir returns 1, call do_ir QUICKLY

	MAJOR ROUTINES:
void init_ir(void)- Initialize the IR Port on timer 2
unsigned char check_ir(void) - Check IR for activity
void do_ir(void)- Handle IR for activity - called once check_ir goes true


    Version      Author   Date           Comments
     1.00           hg.   7/4/02  Initial Development
*/

#define IRLIB 1
//#define DEBUGIR 1 	  // debugging output



/** PROTOTYPES **/
unsigned char check_ir(void);
void do_ir(void);
unsigned int get_ir(void);
void init_ir(void);


unsigned char ir_fun;	  // IR Function Detected
unsigned char ir_exit_delay; // number of tics (3.3ms) before turning back on the interupts.

// These values are from expermentation with debug on
#define IRMIN1	2500	  // MIN start bit  
#define IRMAX1	6500	  // MAX start bit  
#define IRMIN2	2500	  // MIN LOGIC 1 bit  
#define IRTIMEOUT	200	  	 // * 3.3 ms = max time to wait for complete IR  

#define IRMAXLOOKUP 10
const unsigned char ir_lookup[IRMAXLOOKUP] = // lookup values on sony IR
// 0   1   2   3   4   5   6   7   8  9 
{ 68,128, 64,130, 65,132, 66,134, 67, 136};

/* INIT_IR - Initialize the IR Port on timer 2*/
void init_ir(void)
{

	  T2SC= TIM_STOP | TIM_RESET;  // Stop Tim no Prescale
	  T2MODH = 0xFF;
	  T2MODL = 0xFF;   // Set mondlo
	  T2SC0 = 0x4C; // Start Channel - Input capture - both edges INt
	  T2SC= 0;  // start timer

}

/* CHECK_IR - Check IR for activity */
unsigned char check_ir(void)
{
	 if ((T2SC0 & 0x80)) // Input capture!!  
	   return(1);
     return(0);

}


/* DO_IR - Handle IR for activity - called once check_ir goes true */
void do_ir(void)
{
unsigned int irdata[10];  // raw IR data counts
unsigned int txb;
unsigned char xraw;  // incomming raw ir dataa
unsigned char xi;
 
  delay_v= 0x7FFF;
  i=0;
  while ((delay_v) && (i < 10) && !kbhit()) {
	 irdata[i] = get_ir();
	 if (irdata[i] > 0) {
	   if (i == 0) {
	     if (irdata[i] < IRMIN1) return;  // bad start bit 
	   }
	   i++;
	 } 
  }
  if (i > 1) {
#ifdef DEBUGIR  
   for (q=0;q<i;q++)
     printf(".%u",irdata[q]);
#endif	 
   txb = irdata[1];
   if (txb < IRMIN2) return;  // bad start bit
   if (txb > IRMAX1) return;  // bad start bit
   xraw=0;  // clear out the last one
   for (q=2;q<i;q++) {
      if (irdata[q] > 2500)
	    xraw |= 1 << (q-2);
   }
#ifdef DEBUGIR  
     printf("R:%u",xraw);
#endif	 
   // now lookup the function
   for (xi=0;xi<IRMAXLOOKUP;xi++) {
     if (ir_lookup[xi] == xraw) {
	    ir_fun=xi;  // found in table
	 }
   }
  }
  while (delay_v) delay_v--; //wait for rest of cycle - Ignore rest
  xi = T2SC0;	   // read before clear
  T2SC0 = 0x0C; // Clear flag
  T2SC=  TIM_RESET;  // Reset the counter
  T2SC= 0;  // start timer again
#ifdef DEBUGIR  
    printf("fun:%d\n\r",ir_fun);
#endif	 
 }

/* GET_IR - get one transition and return the count */
unsigned int get_ir(void)
{
unsigned int t1;
unsigned char mcp;

    while(!(T2SC0 & 0x80)&& (delay_v)) delay_v--;  // wait fro capture to go set
	mcp = PTD;  // save to check fro low
	t1= ((unsigned int)T2CH0H);
	t1 =  t1  << 8;  // record counts
	t1 += (unsigned int)T2CH0L;
    T2SC0 = 0x0C; // Clear flag
    T2SC=  TIM_RESET;  // Reset the counter
    T2SC= 0;  // start timer again
	// Only record lows!!!
	if (mcp & 0x40)
	  return(t1);	
	else
	  return(0);	
}
