/*   menu.c  - LCD Menu handler
		  
  handles:
    KeyPad/LCD Operation 

	MAJOR ROUTINES:
void handle_keypress(char newkey) - process the keypad / encoder keys  This is the main dispatch to the handlers 
void lcd_MainMenu(void)- reset pointers to main menu (DOES NOT DISPLAY,   JUST RESET POINTERS) 
void Display_Menu(void)- DIsplay any active menus or user entries uses lcdmenumode to determine display This will paint the LCD after an update
void background_lcd(void)- Once per second updates to paint LCD with status 



    Version      Author   Date           Comments
     1.00           hg.   7/4/02  Initial Development
*/

#define MENULIB 1

/** PROTOTYPES **/
void handle_keypress(char newkey);
void handle_menupress(char newkey);
void handle_entrypress(char newkey);
void handle_selectpress(char newkey);
void showlcdentryvalue(void);
void handle_menu_main(void);
void background_lcd(void);
void do_output_menu(void); 
void do_setting_menu(void);
void handle_entry(void); 
void handle_statuspress(char newkey);
void handle_entry_Clock(void); 
void handle_entry_Settings(void); 
void Display_Menu(void);
void handle_entry_Outputs(void);
void handle_eventpgmpress(char newkey);
void exit_prog(void);
void handle_forcepress(char newkey);
void handle_logpress(char newkey);
unsigned char Display_log(char *stmp,char *stmp2);
void EventText( char *stmp, char *stmp2);


extern unsigned char hour, min, sec;  /* current times */


//char lcd1[20];
//char lcd2[20];
unsigned char curloc;	   // curser location

// MENUINg
unsigned char lcdmenumode;       /* menu(1,2) or entry(0) mode  1 = main menu, 2 = sub menu 3 = status display 4=select list*/
unsigned int lcdentryvalue;     /* value for numeric entries */
unsigned char lcdselectlist;     /* value for selection  entries, used for flag in numeric entry */

unsigned char   lcdmenuptr;      /* current menu entry */
unsigned char   lcdmenumenu;     /* current menu table */
unsigned char   lcdmenutimeout;  /* lcd menu timeout - return to main menu when 0 */ 

int limit_hi,limit_lo;  /* Entry Limits */
const char * menuname;			 /* Pointer to the current menu entrytext */

unsigned char prg_event;		 // event being programmed
unsigned char prg_seq;			 // input sequence during event programming

unsigned char backdisplay;		 // background display (status)

/////* MENUING */////
#define MAXMENUENT 6
const char  lcdmenu[5][MAXMENUENT][16] =
{
{"Inputs         ","Outputs        ","Settings       ","Events/Schedule","Clock         ","Log        "},
{"Time           ","Temp           ","X10 Recv       ","Light Sense    ","IR Cmd        ","Inputs     "},
{"Health LED     ","Relay          ","Digital Output ","Tones          ","X10 Send      ","Exp Board  "},
{"Temp Units     ","X10 Address    ","X10 Housecode  ","Relay Mode     ","Logging       ","Load Defaults"},
{"Hour           ","Minute         ","Month          ","Day            ", "Year         ","DOW        "}
};


// SELECT LISTS
#define MAXSELENT 6
const char  selectlist[6][MAXSELENT][7] =
{
{"A-OFF","A-ON","ON","OFF","DIM","BRIGHT"},
{"NO","YES"," "," "},
{"C  ","F "," "," "},
{"NORMAL","ONCE","FLASH"," "},
{"OFF","ON"," "," "},
{"OFF","EVENTS","VALUES","ALL"}
};

// Dummies
//static char lcdmenu[1][1][1];
//static char selectlist[1][1][1];

// define menu modes 
#define LCDMENUMODE_ENTRY  0	  // entry mode 
#define LCDMENUMODE_MAIN   1      // main menu 
#define LCDMENUMODE_SUBMENU 2	  // sub menu
#define LCDMENUMODE_STATUS 3	  // status (inputs) display
#define LCDMENUMODE_SELECT 4	  // select from list mode
#define LCDMENUMODE_TIMEENT 5	  // Time Entry
#define LCDMENUMODE_EVENTPGM 6	  // Events Entry
#define LCDMENUMODE_LOG 7	      // Log display Entry
#define LCDMENUMODE_X10ENTRY 8	  // x10 Address entry mode 
#define LCDMENUMODE_RUNSTATUS 9	  // Run - time status 


/* lCD_MAINMENU - reset pointers to main menu (DOES NOT DISPLAY,   JUST RESET POINTERS) */
void lcd_MainMenu(void)
{

  lcdmenumenu=0;
  lcdmenuptr=0;
  lcdmenumode=LCDMENUMODE_MAIN;
  lcdmenutimeout=20;
  curloc=0;
  backdisplay=0;
}

/* EXIT_PROG - exit the programming mode - save data */
void exit_prog(void)
{

  // move event back up to hoigh ram
  memptr = 0x100 + ((prg_event-1) * sizeof(ev));  // high ram pointer
  // copy it into high mem
  memcpy((void *)memptr,(void *)&ev, sizeof(ev));
  save_event(prg_event-1);
  lcd_MainMenu();
}

/* HANDLE_KEYPRESS - process the keypad / encoder keys */
/* This is the main dispatch to the handlers */
void handle_keypress(char newkey)
{
   lcdmenutimeout=30;  // reset timer
 
   // lcd_clr();
	if (force)
	  handle_forcepress(newkey);
    else if (lcdmenumode == LCDMENUMODE_ENTRY)
      handle_entrypress(newkey);
    else if (lcdmenumode == LCDMENUMODE_X10ENTRY)
      handle_entrypress(newkey);
    else if (lcdmenumode == LCDMENUMODE_TIMEENT)
      handle_entrypress(newkey);
    else if (lcdmenumode == LCDMENUMODE_STATUS)	 /* status display - jump to main menu */
      handle_statuspress(newkey);
    else if (lcdmenumode == LCDMENUMODE_SELECT)
      handle_selectpress(newkey);
    else if (lcdmenumode == LCDMENUMODE_LOG)
      handle_logpress(newkey);
	  
    else if (lcdmenumode == LCDMENUMODE_EVENTPGM)
      handle_eventpgmpress(newkey);
	  
    else
      handle_menupress(newkey);
	// Now display whatever needs to be displayed  
    Display_Menu();
}

// DISPLAY_MENU - DIsplay any active menus or user entries
// uses lcdmenumode to determine display
// This will paint the LCD after an update
void Display_Menu(void) {
char stmp[30];
char stmp2[30];
const char *menupt;

	 // pointer to current mneu/entry item
    menupt=&lcdmenu[lcdmenumenu][lcdmenuptr][0];
  // Line 1	  
	if (force & FORCEHEALTH) {
      sprintf(stmp,"Forcing Health=%d ",healthcode);
	}
	if (force & FORCERELAY) {
      sprintf(stmp,"Forcing Relay, ");
	}
	if (force & FORCEOUTPUT) {
      sprintf(stmp,"Forcing Output, ");
	}
	if (force & FORCETONE) {
      sprintf(stmp,"Forcing Tone, ");
	}
	if (force) {
      sprintf(stmp2,"Press to Abort ");
	}
    else if (lcdmenumode == LCDMENUMODE_ENTRY) { /* entry mode */
      sprintf(stmp,"Enter %s",menupt);
	  if ((lcdmenumenu==3) && (lcdmenuptr == 2)) { // House Code entry
  	    if (lcdentryvalue==0)
          sprintf(stmp2,"OFF");
	    else {
  	      stmp2[0] = 'A';
		  stmp2[0] += (lcdentryvalue-1);
		  stmp2[1] = '\0';
		}
	  }
	  else
        sprintf(stmp2,"%d   ",lcdentryvalue);
    }
    else if (lcdmenumode == LCDMENUMODE_X10ENTRY) { /* X10 addres entry mode */
      sprintf(stmp,"Enter X10 Address");
      sprintf(stmp2,"%c %d   ",(lcdentryvalue >> 4) +  'A', (lcdentryvalue&0XF) +1);
    }
	
    else if (lcdmenumode == LCDMENUMODE_TIMEENT) { /* time mode */
	  if (lcdmenuptr == 5) { // day of week
        sprintf(stmp,"Enter Weekday-> %s",dow_text[lcdentryvalue-1]);
	  }
	  else
        sprintf(stmp,"Enter %.6s -> %d ",menupt,lcdentryvalue);
      sprintf(stmp2,"%2d:%02d.%02d %2d/%02d/%02d ",hour,min,sec, month,day,year);
    }
    else if (lcdmenumode == LCDMENUMODE_STATUS) { /* Stats mode */
      sprintf(stmp,"Status %s",menupt);
      sprintf(stmp2,"Waiting...");
    }
    else if (lcdmenumode == LCDMENUMODE_SELECT) { /* select formlist mode */
      sprintf(stmp,"Select %s",menupt);
      sprintf(stmp2,"%s",&selectlist[lcdselectlist][lcdentryvalue][0]);
    }
    else if (lcdmenumode == LCDMENUMODE_EVENTPGM) { /* programming mode */
	  EventText(stmp,stmp2);
	}
    else if (lcdmenumode == LCDMENUMODE_LOG) { /* select formlist mode */
	  Display_log(stmp,stmp2);

    }
    else if (lcdmenumode == LCDMENUMODE_MAIN) { /* select formlist mode */
	  sprintf(stmp," ** Main Menu **");
      sprintf(stmp2,menupt);
    }
    else {  /* menu mode */
      sprintf(stmp,"* Menu:%s",menuname);
      sprintf(stmp2,menupt);
    }
    lcd_putl(stmp);
    lcd_putl2(stmp2);

    if (curloc > 0) {
	   lcd_goto(curloc);
	   //lcd_cursor();
	}

 }      
 
// Built the text for an eve t to display	  
void EventText(char *stmp, char *stmp2)
{
unsigned char rr,rx;
char *np;
const char *cnp;

  	rr=prg_event;
	  if (((prg_seq==3) || (prg_seq==4))&&(ev.in1_type==INPUTTIME)) {  // time program - shift to make room
        sprintf(stmp,"IF %.4s ",input_names[ev.in1_type]);
		np=&stmp[8];
        sprintf(np,"%d:%sto%d:%s",(ev.in1_val[0]>>2),min_sel_text[(ev.in1_val[0] & 3)],ev.in1_val[1] >> 2,min_sel_text[(ev.in1_val[1] & 3)]);
	  }
	  else {
	    if (rr < 10)
         rx = '0' + rr;
		else
         rx = 'A' + (rr-10);
		
        sprintf(stmp,"%c IF %.4s ",rx,input_names[ev.in1_type]);
		np=&stmp[10];
  	    if (ev.in1_type==INPUTTIME)   // TIME INPUT
          sprintf(np,"%d:%sto%d",(ev.in1_val[0] >> 2),min_sel_text[(ev.in1_val[0] & 3)],ev.in1_val[1] >> 2);
	    else if ((ev.in1_type==INPUTX10) && (ev.in1_val[1] < 6))   // X10 INPUT
	      sprintf(np,"%d-%.4s",ev.in1_val[0],selectlist[0][ev.in1_val[1]]);
	    else if ((ev.in1_type==INPUTIR))   // IR INPUT OR DELAY
	      sprintf(np,"%d",ev.in1_val[0]);
	    else if ((ev.in1_type==INPUTDIGITAL) && (ev.in1_val[1] < 2))   // DIGITAL  INPUTs
	      sprintf(np,"%d-%s",ev.in1_val[0],selectlist[4][ev.in1_val[1]]);
	    else if (ev.in1_type==INPUTDOW) {   // dow
		  if ((ev.in1_val[0] < 10) && (ev.in1_val[0] > 0)) 
	        sprintf(np,"%s",dow_text[ev.in1_val[0]-1]);
		  else {
		    if (ev.in1_val[0] > 0)
	          sprintf(np,"%d",ev.in1_val[0]-9);
			else
	          sprintf(np," ");
		  }
		}  
	    else    // TEMP and LIGHT INPUT and delays and  ANALOG  INPUTs
	      sprintf(np,"%dto%d",ev.in1_val[0],ev.in1_val[1]);
      } 
  // Line 2
     cnp = &output_names[ev.out1_type][0];
  	  if (ev.in_logic == 0)
        sprintf(stmp2,"THEN %.6s ",cnp);
  	  else if ((ev.in_logic >0)&&  (ev.in_logic <=NUMEVENTS)) {
	    rx = ev.in_logic;
		if (rx > 9) rx += 'A'-10;
		else rx += '0';
        sprintf(stmp2,"AND%c %.6s ",rx,cnp);
	  }
  	  else {
	    rx = ev.in_logic-NUMEVENTS;
		if (rx > 9) rx += 'A'-10;
		else rx += '0';
        sprintf(stmp2,"OR%c  %.6s ",rx,cnp);
      }
	  if ((ev.out1_type==OUTPUTTONE)|| (ev.out1_type==OUTPUTDLYS)|| (ev.out1_type==OUTPUTDLYM)) // tone or delay
	    sprintf(&stmp2[12],"%d",ev.out1_val[0]);
	  else if ((ev.out1_type==OUTPUTX10) && (ev.out1_val[1] < 6))
	    sprintf(&stmp2[12],"%c%d-%s",(ev.out1_val[0] >> 4) +  'A', ((ev.out1_val[0]&0XF) +1),selectlist[0][ev.out1_val[1]]);
	  else if (((ev.out1_type == OUTPUTOUTPUT)||(ev.out1_type == OUTPUTEXP)) && (ev.out1_val[1] < 2))
	    sprintf(&stmp2[12],"%d-%s",ev.out1_val[0],selectlist[4][ev.out1_val[1]]);
	  else if ((ev.out1_type == OUTPUTRELAY) && (ev.out1_val[1] < 2))
	    sprintf(&stmp2[12],"%d-%s",ev.out1_val[0],selectlist[4][ev.out1_val[1]]);
	  else if (ev.out1_type == OUTPUTHEALTH)
	    sprintf(&stmp2[12],"%d",ev.out1_val[0]);
	  else if (ev.out1_type == OUTPUTFLAG) {
	    if (ev.out1_val[0]==10) {
		   stmp2[12]='+';
		   stmp2[13]=0;
		 }
		 else if (ev.out1_val[0]==11) {
		   stmp2[12]='-';
		   stmp2[13]=0;
		 } 
		 else
    	    sprintf(&stmp2[12],"%d",ev.out1_val[0]);
	  }
	  else if (ev.out1_val[0] < 2)
	    sprintf(&stmp2[12],"%s",selectlist[4][ev.out1_val[0]]);
	  else
	    sprintf(&stmp2[12],"%d-%d",ev.out1_val[0],ev.out1_val[1]);
}

/* DISPLAY_LOG - formats the log buffer up */
/* Retunrs: 1 = end of log, 0 */ 
unsigned char Display_log(char *stmp,char *stmp2)
{
unsigned int zh,zm,zmm,zd;


      if (logstart) {  // potential roll over
	    // i is EEPROM address
	    i = (logstart + (lcdentryvalue * 6));
  	    //printf("~%d",i);
        if (i+7 > EEPROMSIZE) { // logging limit?? roll over to bottom of log
		  i += 12; // last entry space not used
		  i -= (EEPROMSIZE+1);  // now I'm relative to bottom
		  i = i / 6;
		  i = i * 6;  // need even multiple of 6 
		  i += logfirst; // add some
		  i += 4;		 // past pointers
		}
  	    //printf("!%d~",i);
	  }
	  else {
   	    i = (lcdentryvalue*6) + logfirst+4;
	  }
  	    //printf("!%d~",i);
	  // pull out word integer value
      q = 	eeprom_read(i++);
      zd = 	eeprom_read(i++);
      q = q + (zd << 8);
	  // zd = log entry number, zh = hour/dtype zm = dtype
	  zd = lcdentryvalue+1;
	  zh = eeprom_read(i++);
	  zm = zh >> 5;  // upper nibble has data type
      if (q == 0xFFFF)   {  // ENd Flag  or mess
	        sprintf(stmp,"Log END  ");
		    stmp2[0]=0;
		    limit_hi = lcdentryvalue;
		    return(1);
      }
	  if (zm==LOG_EVENT) {
		  if (q <= NUMEVENTS) 
            sprintf(stmp,"Log:%d Event:%d",zd,q);
  	      else if (q <= (NUMEVENTS*2)) 
            sprintf(stmp,"Log:%d Ev Off:%d",zd,q-NUMEVENTS);
	  }
	  else if (zm==LOG_VALUE_TEMP) 
          sprintf(stmp,"Log:%d Temp %d.%d%c",zd,conv_temp(q),conv_temp_dp(q),(tempunits == 0)?'C':'F');
	  else if (zm==LOG_VALUE_TEMPB) 
          sprintf(stmp,"Log:%d TmpB %d.%d%c",zd,conv_temp(q),conv_temp_dp(q),(tempunits == 0)?'C':'F');
	  else if (zm==LOG_VALUE_X10) 
          sprintf(stmp,"Log:%d X10%d.%s",zd,q>>4,selectlist[0][q&0xf]);
	  else if (zm==LOG_VALUE_LIGHT) 
          sprintf(stmp,"Log:%d Light %d",zd,q);
	  else if (zm==LOG_VALUE_ANLG) 
          sprintf(stmp,"Log:%d Anlg %d",zd,q);
	  else if (q == 0xFFFE) 
            sprintf(stmp,"Log:%d Unit Start",zd);
	  else if (q == 0xFFFD) 
            sprintf(stmp,"Log:%d Time Sync",zd);
	  else 
	         sprintf(stmp,"Log:%d  %d",zd,zm);
  	   
      // zh = hour, zd = day, zm = min
	  zh = zh & 0x1F;  // clear upper nibble
	  zm = eeprom_read(i++);
	  zmm = eeprom_read(i++);
	  zd = eeprom_read(i++);
      sprintf(stmp2,"At %d:%02d %d/%d",zh,zm,zmm,zd);
	  limit_hi = lcdentryvalue+2;
	  return(0);
}

/* HANDLE_EVENTPGMPRESS Event  PROGRAMMING inputs */ 
void handle_eventpgmpress(char newkey)
{
unsigned char *datpt;   // temp pointer to teh data we are mess'n with
unsigned char xtype; 
unsigned char ytype; 

  // Handle enter forst
  if (newkey == ENTER) {
    lcdselectlist=0;   // used as flag for numeric entry
    if (prg_seq < 7) {
      prg_seq++;
    }
    else
      exit_prog();       
  }
  // setup datapointer and limits
  xtype = ev.in1_type;
  ytype = ev.out1_type;
  if (prg_seq==0) {
    // Input the event number to program
     datpt = &prg_event;
     limit_hi=NUMEVENTS,limit_lo=1;
  }
  if (prg_seq==1) {
   // input type entry 
   datpt = &ev.in1_type,limit_hi=NUMINPUTTYPES-1,limit_lo=0;
   if (ds1822_num==2)  // two temp sensors
     limit_hi=NUMINPUTTYPES,limit_lo=0;  /* Entry Limits */
   else
     limit_hi=(NUMINPUTTYPES-1),limit_lo=0;  /* Entry Limits */
   curloc=5;
  }
  if (prg_seq==2) {			 // input value
    if (xtype == 0) 
     exit_prog();  // jump out
    curloc=10;
    if  (xtype == INPUTTIME)
     limit_hi=95,limit_lo=0;
    else if ((xtype ==INPUTTEMP) || (xtype ==INPUTTEMP2)) {
	 if (tempunits == 0)  // C
       limit_hi=125,limit_lo=0;
	 else 
       limit_hi=250,limit_lo=0;
	}
    else if (xtype ==INPUTX10)
     limit_hi=15,limit_lo=1;
    else if (xtype ==INPUTDOW)
     limit_hi=40,limit_lo=1;
    else if (xtype ==INPUTLIGHT)
     limit_hi=100,limit_lo=0;
    else if ((xtype ==INPUTDLYS)||(xtype ==INPUTDLYM))
     limit_hi=255,limit_lo=0;
    else if (xtype ==INPUTIR)
     limit_hi=9,limit_lo=1;
    else if (xtype ==INPUTFLAG)
     limit_hi=9,limit_lo=0;
    else  // analog?
     limit_hi=255,limit_lo=0;
    datpt = &ev.in1_val[0];
    if ((newkey == ENTER)&& (*datpt == 0)) { // Just comming in set the defaults
      if  (xtype == INPUTTEMP) 
	    *datpt = conv_temp(curtemp);
      else if (xtype ==INPUTLIGHT)
	    *datpt =  lightavgsum >> 3;

	} 
  }
  if (prg_seq==3) {  // second val input
    curloc=14;
    if  (xtype == INPUTTIME)
     limit_hi=95,limit_lo=0;
    else if ((xtype ==INPUTTEMP)|| (xtype ==INPUTTEMP2))
     limit_hi=250,limit_lo=ev.in1_val[0];
    else if (xtype ==INPUTX10)
     limit_hi=5,limit_lo=0;
    else if (xtype ==INPUTDOW)
     prg_seq++;
    else if (xtype ==INPUTDIGITAL)
     limit_hi=1,limit_lo=0;
    else if (xtype ==INPUTFLAG)
     limit_hi=9,limit_lo=ev.in1_val[0];
    else if (xtype ==INPUTANALOG)
     limit_hi=255,limit_lo=ev.in1_val[0];
    else if ((xtype ==INPUTDLYS)||(xtype ==INPUTDLYM))
     limit_hi=255,limit_lo=ev.in1_val[0];
    else if (xtype ==INPUTIR) // Skip this 
     prg_seq++;
    else if (xtype ==INPUTLIGHT)
     limit_hi=100,limit_lo=ev.in1_val[0];
    else 
     limit_hi=4,limit_lo=1;
    datpt = &ev.in1_val[1];
    if ((newkey == ENTER)&& (*datpt == 0)) { // Just comming in set the defaults
      if  (xtype == INPUTTIME) 
	    *datpt = ev.in1_val[0];
	} 
  }
  if (prg_seq==4) {  // Logix (and/or/then) val input
    curloc=0x40;
    datpt = &ev.in_logic;
    limit_hi=(NUMEVENTS*2),limit_lo=0;
	
  }
  if (prg_seq==5){   // Output Type
   curloc=0x45;
   datpt = &ev.out1_type,limit_hi=NUMOUTPUTTYPES-1,limit_lo=0;
   }
  if (prg_seq==6) {	// first put Value								
    curloc=0x4C;
    if (ytype ==0)  // no more
     exit_prog();
    else if  (ytype == OUTPUTHEALTH)
     limit_hi=4,limit_lo=0;
    else if (ytype ==OUTPUTRELAY) {
	 if (ev.out1_val[1] > 0) ev.out1_val[1]=1;
     limit_hi=2,limit_lo=1;
	}
    else if (ytype ==OUTPUTOUTPUT)
     limit_hi=4,limit_lo=1;
    else if (ytype ==OUTPUTEXP)
     limit_hi=8,limit_lo=1;
    else if (ytype ==OUTPUTTONE)
     limit_hi=9,limit_lo=1;
    else if (ytype ==OUTPUTFLAG)
     limit_hi=11,limit_lo=0;
    else if ((ytype ==OUTPUTDLYS)||(ytype ==OUTPUTDLYM))
     limit_hi=255,limit_lo=1;
    else   // X10??
     limit_hi=255,limit_lo=0;
    datpt = &ev.out1_val[0];
  }
  if (prg_seq==7) {	// second oyutput value
   curloc=0x4F;
    if  ((ytype == OUTPUTHEALTH)|| (ytype == OUTPUTDLYS)|| (ytype == OUTPUTDLYM)|| (ytype == OUTPUTFLAG))
     exit_prog();
    else if (ytype ==OUTPUTRELAY)
     limit_hi=1,limit_lo=0;
    else if ((ytype ==OUTPUTOUTPUT)||(ytype ==OUTPUTEXP))
     limit_hi=1,limit_lo=0;
    else if (ytype ==OUTPUTTONE)
     exit_prog();
    else   // X10??
     limit_hi=6,limit_lo=0;
    datpt = &ev.out1_val[1];
  }

   
  // handle user event 
  if (newkey == RIGHTUP) {
      if (*datpt < limit_hi) {
        (*datpt)++;
	  }
  }
  else if (newkey == LEFTDOWN) {
      if (*datpt > limit_lo) { 
        (*datpt)--;
	  }
  }
  else if (newkey == EXIT) {
    lcd_MainMenu();
    //exit_prog();       
  }
  if ((newkey >= '0') && (newkey <='9')) { // numeric entry
    newkey=(newkey - '0');
    if (lcdselectlist) {
      *datpt = *datpt * 10;   // Assume two digit entry
      *datpt += newkey;
      if (*datpt > limit_hi) 
        *datpt = newkey;
	} 
	else {  // first time thorugh  - assume 1 digit entry 
      *datpt = newkey;
  	  lcdselectlist=1; // set flag for second time
	}
  }
  else {
    // safety check
    if (*datpt > limit_hi) *datpt=limit_hi;
    if (*datpt < limit_lo) *datpt=limit_lo;
  }
  if (prg_seq==0) {
    // COPY it into the local editing buffer
	 memptr = 0x100 + ((prg_event-1) * sizeof(ev));  // high ram pointer
   	 // copy it FROM high mem
 	  memcpy((void *)&ev,(void *)memptr, sizeof(ev));
  }

}

/* HANDLE_STATUSPRESS - handle keypad in status (input) mode */
void handle_statuspress(char newkey)
{
  if (newkey == RIGHTUP) {
    if (lcdmenuptr < MAXMENUENT-1) {
      if (lcdmenu[lcdmenumenu][lcdmenuptr+1][0] > ' ') 
        lcdmenuptr++;
      else
        lcdmenuptr=0;
	  // skip if no x10
	  if ((lcdmenuptr == (INPUTX10-1)) && (X10ok != TRUE)) lcdmenuptr++;
    }
    else {
      lcdmenuptr=0;
      lcd_clr();
	  menuname = &lcdmenu[lcdmenumenu][lcdmenuptr][0];
    }
	backdisplay= lcdmenuptr+1;
  }
  else if (newkey == LEFTDOWN) {
    if (lcdmenuptr >  0) {  // Move down
      if (lcdmenu[lcdmenumenu][lcdmenuptr-1][0] > ' ') 
        lcdmenuptr--;
      else {
        lcdmenuptr=MAXMENUENT-1;// goto last real entry 
		while (lcdmenu[lcdmenumenu][lcdmenuptr][0] == ' ') lcdmenuptr--;
	  }
	  // skip if no x10
	  if ((lcdmenuptr == (INPUTX10-1)) && (X10ok != TRUE)) lcdmenuptr--;
    }
    else {
      lcdmenuptr=MAXMENUENT-1;  // goto last real entry 
  	  while (lcdmenu[lcdmenumenu][lcdmenuptr][0] == ' ') lcdmenuptr--;
	}
    lcd_clr();
    menuname = &lcdmenu[lcdmenumenu][lcdmenuptr][0];
	backdisplay= lcdmenuptr+1;  // setup for background display of status
  }
  else if (newkey == EXIT) {
    lcd_MainMenu();       
  }
}

/* HANDLE_LOGPRESS - handle keypad in log view mode */
void handle_logpress(char newkey)
{
  if (newkey == RIGHTUP) {
    if (lcdentryvalue < limit_hi) 
      lcdentryvalue++;
  }
  else if (newkey == LEFTDOWN) {
    if (lcdentryvalue > limit_lo) 
      lcdentryvalue--;
  }
  else if (newkey == EXIT) {
    lcd_MainMenu();       
  }
}

/* HANDLE_FORCEPRESS - handle keypad in force mode */
void handle_forcepress(char newkey)
{
  if ((newkey == EXIT) || (newkey == ENTER)) {
    force=0;
    lcd_MainMenu();       
  }
}


/* HANDLE_MENUPRESS - handle keypad in menu mode */
void handle_menupress(char newkey)
{

  if (newkey == RIGHTUP) {
    if (lcdmenuptr < MAXMENUENT-1) {
      if (lcdmenu[lcdmenumenu][lcdmenuptr+1][0] > ' ') 
        lcdmenuptr++;
      else
        lcdmenuptr=0;
    }
    else
      lcdmenuptr=0;
  }
  else if (newkey == LEFTDOWN) {
    if (lcdmenuptr >  0) {  // Move down
      if (lcdmenu[lcdmenumenu][lcdmenuptr-1][0] > ' ') 
        lcdmenuptr--;
      else {
        lcdmenuptr=MAXMENUENT-1;// goto last real entry 
		while (lcdmenu[lcdmenumenu][lcdmenuptr][0] == ' ') lcdmenuptr--;
	  }
    }
    else {
      lcdmenuptr=MAXMENUENT-1;  // goto last real entry 
  	  while (lcdmenu[lcdmenumenu][lcdmenuptr][0] == ' ') lcdmenuptr--;
	}
  }
  else if (newkey == 'H') {
     lcd_clr();
     lcd_putl("Arrow keys to scroll");
     lcd_putl2("Use Enter to select");
  }
  else if (newkey == EXIT) {
    lcd_MainMenu();       
  }
  else if (newkey == ENTER) {
//    printf("Menu Entry %d: %d\n\r",lcdmenumenu,lcdmenuptr);
 
    menuname =&lcdmenu[lcdmenumenu][lcdmenuptr][0];
    switch (lcdmenumenu) {
      case 0:  /* Main Menu */
	    handle_menu_main();
        break;
       case 1:  /* Inputs Menu */
          lcdmenumode=LCDMENUMODE_STATUS;
          break;
       case 2:  /* Outputs Menu */
  	      do_output_menu();
		  break;
       case 3:  /* Settings Menu */
	       do_setting_menu();
          break;
       case 4:  
            break;


    }
  }
}


// HANDLE_MENU_MAIN
void handle_menu_main(void)
{
        switch (lcdmenuptr) {
          case 0: /* set inputs */
            lcdmenumode = LCDMENUMODE_STATUS;
            lcdmenumenu =1;
	     	lcdmenuptr=0;
 	        x10_fun=0;
			backdisplay=1;
            break;
          case 1: /* Set Outputs */
            lcdmenumode=LCDMENUMODE_SUBMENU;
            lcdmenumenu =2;
	        lcdmenuptr=0;
	        break;
          case 2: /* settings */
            lcdmenumode=LCDMENUMODE_SUBMENU;
            lcdmenumenu =3;
	        lcdmenuptr=0;
            break;
          case 3: /* Events */
            lcdmenumode=LCDMENUMODE_EVENTPGM;
 	        curloc=0;
		    prg_seq=0;
		    prg_event=1;
		    // COPY it into the local editing buffer
		    memptr = 0x100 + ((prg_event-1) * sizeof(ev));  // high ram pointer
   		    // copy it FROM high mem
 	 	    memcpy((void *)&ev,(void *)memptr, sizeof(ev));
			limit_hi=NUMEVENTS,limit_lo=1;  /* Entry Limits */
            break;
          case 4: /* clock */
            lcdmenumode=LCDMENUMODE_TIMEENT;
            lcdmenumenu =4;
	        lcdmenuptr=0;
            lcdentryvalue=hour;
			limit_hi=23,limit_lo=0;  /* Entry Limits */
            break;
          case 5: /* loggin */
            lcdmenumode=LCDMENUMODE_LOG;
			limit_lo=0;
			limit_hi=1;
			lcdentryvalue=0;
            break;
          default:
            lcdmenumode=LCDMENUMODE_MAIN;
            lcdmenumenu =0;
	        lcdmenuptr=0;
            break;
        }
}


void do_output_menu(void) 
{
        switch (lcdmenuptr) {
          case 0: /* Health Code */
            lcdmenumode=LCDMENUMODE_ENTRY;
            lcdentryvalue=healthcode;
			limit_hi=4,limit_lo=0;  /* Entry Limits */
            break;
          case 1: /* Relays */
            lcdmenumode=LCDMENUMODE_ENTRY;
			lcdentryvalue=relay+1;
			limit_hi=2,limit_lo=1;  /* Entry Limits */
            break;
          case 2: /* Digitial Outs */
            lcdmenumode=LCDMENUMODE_ENTRY;
			lcdentryvalue=1;
			limit_hi=4,limit_lo=1;  /* Entry Limits */
            break;
          case 3: /* Tones Outs */
            lcdmenumode=LCDMENUMODE_ENTRY;
			lcdentryvalue=1;
			limit_hi=9,limit_lo=1;  /* Entry Limits */
            break;
          case 4: /* X10 Send */
            lcdmenumode=LCDMENUMODE_X10ENTRY; // get address
			lcdentryvalue=x10_sendaddr;
			limit_hi=255,limit_lo=0;  /* Entry Limits */
            break;
          case 5: /* Expansion Outs */
            lcdmenumode=LCDMENUMODE_ENTRY;
			lcdentryvalue=1;
			limit_hi=8,limit_lo=1;  /* Entry Limits */
            break;
		}
}


void do_setting_menu(void)
{
		 
          switch (lcdmenuptr) {
            case 0: /* temp units */
              lcdmenumode=LCDMENUMODE_SELECT;
              lcdentryvalue=tempunits;
              lcdselectlist=2;
			  limit_hi=1,limit_lo=0;  /* Entry Limits */
             break;
            case 1: /* X10 Addess */
              lcdmenumode=LCDMENUMODE_ENTRY;
              lcdentryvalue=main_x10_addr+1;
			  limit_hi=16,limit_lo=1;  /* Entry Limits */
             break;
            case 2: /* X10 HC */
              lcdmenumode=LCDMENUMODE_ENTRY;
			  if (main_x10_hc>15)
                lcdentryvalue=0;
			  else
                lcdentryvalue=main_x10_hc+1;
			  limit_hi=15,limit_lo=0;  /* Entry Limits */
             break;
            case 3: /* relay Mode */
              lcdmenumode=LCDMENUMODE_SELECT;
              lcdselectlist=3;
              lcdentryvalue=relaymode;
			  limit_hi=2,limit_lo=0;  /* Entry Limits */
             break;
            case 4: /* Logging */
              lcdmenumode=LCDMENUMODE_SELECT;
              lcdselectlist=5;
              lcdentryvalue=logging;
			  limit_hi=3,limit_lo=0;  /* Entry Limits */
             break;
            case 5: /* Load Default */
              lcdmenumode=LCDMENUMODE_SELECT;
              lcdselectlist=1;
              lcdentryvalue=0;
			  limit_hi=1,limit_lo=0;  /* Entry Limits */
             break;
            default:
              lcdmenumode=LCDMENUMODE_MAIN;
              lcdmenumenu =0;
              break;
          }
		}


/* HANDLE_ENTRYPRESS - handle keypad in entry mode */
void handle_entrypress(char newkey)
{
  if (newkey == RIGHTUP) {
    if (lcdentryvalue < limit_hi) 
      lcdentryvalue++;
  }
  else if (newkey == LEFTDOWN) {
    if (lcdentryvalue > limit_lo) 
      lcdentryvalue--;
  }
  else if (newkey == EXIT) {
    lcd_MainMenu();       
  }
  #ifdef KEYPAD
  
  else if ((newkey >= '0') && (newkey <='9')) {
    lcdentryvalue = lcdentryvalue * 10;
    lcdentryvalue += (newkey - '0');
    if (lcdentryvalue > limit_hi) 
      lcdentryvalue = (newkey - '0');
  }
  #endif
  else if (newkey == ENTER) {
     handle_entry();
  }

}




/* HANDLE_SELECTPRESS - handle keypad in select from list mode */
void handle_selectpress(char newkey)
{
  if (newkey == RIGHTUP) {
    if (lcdentryvalue < MAXSELENT-1)  {
      if (selectlist[lcdselectlist][lcdentryvalue+1][0] > ' ') 
        lcdentryvalue++;
      else
	 lcdentryvalue=0;
    }
    else
      lcdentryvalue=0;
  }
  else if (newkey == LEFTDOWN) {
    if (lcdentryvalue > 0)  {
        lcdentryvalue--;
    }
  }
  else if (newkey == EXIT) {
    lcd_MainMenu();       
  }
  else if (newkey == 'H') {
     lcd_clr();
     lcd_putl("Sorry no help avail.");
     lcd_putl2("Use Up/Dn to select");
  }
  else if ((newkey >= '0') && (newkey <='9')) {
    lcdentryvalue = (newkey - '0');
    if (lcdentryvalue > 3) 
      lcdentryvalue = 3;
  }
  else if (newkey == ENTER) {
     handle_entry();
  }
}


/* HANDLE_ENTRY - this handles and entry Dispatch after select or numeric enter */
void handle_entry(void) 
{

     /* handle the entry - store or save*/
     switch(lcdmenumenu) {
       case 4:  /* time Entries */
	     handle_entry_Clock(); 
         break;
 	   case 2:  /* Outputs */
	     handle_entry_Outputs();
	     break;
	  case 3:  /* settingsentries */
	      handle_entry_Settings();
		  lcd_MainMenu();
	      break;
	  case 0:  /* MAIN Menu   */
        break;
     }

}

/* HANDLE_ENTRY_OUTPUTS - */
void handle_entry_Outputs(void)
{
unsigned char ad,hc;
		 
         switch (lcdmenuptr) {
           case 0:      // health UInput
		     force |= FORCEHEALTH;
			 healthcode=lcdentryvalue;
			 lcd_MainMenu();
  	         break;
           case 1: 					  // Relay
		     if (lcdmenumode == LCDMENUMODE_ENTRY) {  // Relay Sleected - on or off
              lcdmenumode=LCDMENUMODE_SELECT;
              lcdselectlist=4;
			  relay=lcdentryvalue-1;
			  lcdentryvalue=1;
			 }
		     else {
		      force |= FORCERELAY;
			  set_relay(relay,lcdentryvalue);
			  lcd_MainMenu();
			 }
			 break;
           case 2: 					  // Digital Output
	       case 5: 					  // Expansion Output
	         if (lcdmenumode == LCDMENUMODE_ENTRY) {  // Selected - on or off
              lcdmenumode=LCDMENUMODE_SELECT;
              lcdselectlist=4;
			  if (lcdmenuptr==2) digitalout=2 << (lcdentryvalue);
			  else digitalout=0x80 >> (lcdentryvalue-1);
			  lcdentryvalue=1;
			 }
		     else {
		      force |= FORCEOUTPUT;
			  if (lcdentryvalue) {  // ON
			    if (lcdmenuptr==2) kp_io_reg2 |= digitalout;
				else exp_io_reg |= digitalout;
			  }
			  else { 		   // OFF 
			    if (lcdmenuptr==2) kp_io_reg2 &= ~digitalout;
				else exp_io_reg &= ~digitalout;
			  }
			  lcd_MainMenu();
			 }
			 break;
           case 3: 					  // Tones
			 i=lcdentryvalue;
 	         force |= FORCETONE;
			 set_tone(i);
		 	 //buzz(i,i * 500);
			  lcd_MainMenu();
			 break;
           case 4: 					  // X10
		     if (lcdmenumode == LCDMENUMODE_X10ENTRY) {  // X10 Address Sleected - now function
              lcdmenumode=LCDMENUMODE_SELECT;
              lcdselectlist=0;
			  x10_sendaddr=lcdentryvalue;
			  lcdentryvalue=2;  // start with ON as default
			 }
		     else {
			   ad=(x10_sendaddr & 0XF);
			   hc = x10_sendaddr >> 4;
 		       send_X10(graycode[ad] << 1,hc);		  // send address
			   // send function
		       send_X10((lcdentryvalue*2+1),hc);   // function
			  lcd_MainMenu();
			 }  
			 break;
         }
}		 


// HANDLE_ENTRY_SETTINGS - handle enter press on time entries
void handle_entry_Settings(void) 
{
         switch (lcdmenuptr) {
           case 1:			  // X10 Address
			main_x10_addr =lcdentryvalue-1;
			break;
           case 2:		  				   // X10 House Code
		    if (lcdentryvalue)
			  main_x10_hc =lcdentryvalue-1;
			else
			  main_x10_hc=25;  // Make it Z
			break;
           case 3:						 // RElay Mode
			relaymode =lcdentryvalue;
			break;
           case 4:
			logging =lcdentryvalue;
			if (logging != logtype) {  // just turned on - clear log
			   log_reset();
			   logtype=logging;
			}
			break;
           case 0:
	        tempunits =lcdentryvalue;
  	        break;
           case 5:
  		    if (lcdentryvalue==1) {
			  default_parms();
			}
  	        break;
          }
		  // save everything to eeprom
		  save_parms();
		  
}
// HANDLE_ENTRY_CLOCK - handle enter press on time entries
void handle_entry_Clock(void) 
{
         sclk_Write(SCLK_SEC,0); // assume 0 secs at press
         switch (lcdmenuptr) {
           case 0:			 // hours enters - jump to minutes
             sclk_Write(SCLK_HOUR,to_BCD(lcdentryvalue));
             lcdentryvalue=min;
			 limit_hi=59,limit_lo=0;  /* Entry Limits */
			 lcdmenuptr++;
             break;
           case 1: 		  // minutes entered - goto month
             sclk_Write(SCLK_MIN,to_BCD(lcdentryvalue));
             lcdentryvalue=month;
			 limit_hi=12,limit_lo=1;  /* Entry Limits */
			 lcdmenuptr++;
             break;
           case 2:  // month entered go to day
             sclk_Write(SCLK_MONTH,to_BCD(lcdentryvalue));
             lcdentryvalue=day;
			  limit_hi=31,limit_lo=1;  /* Entry Limits */
			 lcdmenuptr++;
             break;
           case 3:  // day entered got o year
             sclk_Write(SCLK_DAY,to_BCD(lcdentryvalue));
             lcdentryvalue=year;
			  limit_hi=99,limit_lo=1;  /* Entry Limits */
			 lcdmenuptr++;
             break;
           case 4:  // day entered got o year
              sclk_Write(SCLK_YEAR,to_BCD(lcdentryvalue));
             lcdentryvalue=dow;
			  limit_hi=7,limit_lo=1;  /* Entry Limits */
		     lcdmenuptr++;
             break;
           case 5:  // day or week entered got o year
             sclk_Write(SCLK_DOW,to_BCD(lcdentryvalue));
		     lcd_MainMenu();
             break;
        }
         Read_Time();	/* get current values */
}	   







/* BACKGROUND_LCD - Once per second updates to paint LCD with status */
void background_lcd(void)
{
unsigned char a1,a2,a3,a4;
char stmp[30];

    if (lcdmenumode==LCDMENUMODE_RUNSTATUS) {
      sprintf(stmp,"Running.. %d Ev Act",numfired);
      lcd_putl(stmp);
    }
    if (backdisplay>0) {
	    stmp[0] = 0;
		// format up a buffer with the current data
		if (lcdmenumode == LCDMENUMODE_STATUS) { // refresh every sec when in status mode
		  if (sec & 2) {  // alternate display every other sec (temp and dig/analog ins)
		    if (backdisplay==INPUTDIGITAL) backdisplay=INPUTANALOG;
		    if (backdisplay==INPUTTEMP) backdisplay=INPUTTEMP2;
		  }
		  else {
		    if (backdisplay==INPUTANALOG) backdisplay=INPUTDIGITAL;
		    if (backdisplay==INPUTTEMP2) backdisplay=INPUTTEMP;
		  }
		  do_status(backdisplay,stmp,1);
		}
		else  // just background
		  do_status(backdisplay,stmp,0);
	    if (stmp[0]) 
          lcd_putl2(stmp);
    }	 
}




