/*   rulez.c  - Simple rulez processing

  handles:
    Background rules (Events) processing


    Version      Author   Date           Comments
     1.00           hg.   7/7/2  Initial Development
*/
/** PROTOTYPES **/
void rule_process(void);
unsigned char evaluate(unsigned char r);
void do_rule(unsigned char r);
unsigned char get_fired(unsigned char r);
void set_fired(unsigned char r,unsigned char val);

// fired flags accessed as bits
unsigned char fired[2];

unsigned char disp_rule;  // current rule to display 
unsigned char newtone;	  // new commanded tone

unsigned  char newIR;	  // new commanded tone


/* get_fired - return fired flag from bit field */
unsigned char get_fired(unsigned char r)
{
  if (r < 8) 
  	 return(fired[0] & (1 << r)?1:0);
  else {
     r -=8;
  	 return(fired[1] & (1 << r)?1:0);
  }
}



/* set_fired - set fired flag */
void set_fired(unsigned char r,unsigned char val)
{
unsigned char *x;

  if (r < 8) 
  	 x = &fired[0];
  else {
     r-=8;
  	 x = &fired[1];
  }
  if (val) 
    *x |= 1 << r;
  else
    *x &= ~(1 << r);
}

/* RULE_PROCESS - Background rules processing - called from one_sec */
void rule_process()
{
unsigned char r;
unsigned char high_r;  // highest rule
unsigned int memptr;
unsigned char xnumfired; 
 
  // if in programming mode - dn't process rules (they are using the ev buffer)
  if (lcdmenumode==LCDMENUMODE_EVENTPGM)
    return;
  // start with defaults - relays off unless forcing
  if (!(force & FORCEHEALTH))
    healthcode=0;
  if (!(force& FORCERELAY))
    kp_io_reg &= ~(KIO_RELAY + KIO_RELAY2);
  if (!(force& FORCEOUTPUT)) 
    exp_io_reg=0,kp_io_reg2 &= 0xC3;  // clear digital IO
  xnumfired=0;
  // loop thru all the events	
  memptr = 0x100;  // high ram pointer
  high_r=0;
  newtone=0;
  newIR =ir_fun;   // ok get any new IR function from the IR (ir_handler)
  if (ir_fun) {
    lx_ir_fun=ir_fun;
  }
  ir_fun=0; 		 // clear out we only handle once
  // bump the next rult to display on the status display - if higher than the last event then back to zero
  for (r=0;r<NUMEVENTS;r++) {
    // move in the rule for processing
    // copy it FROM high mem
    memcpy((void *)&ev,(void *)memptr, sizeof(ev));
    memptr +=sizeof(ev);
    if (ev.in1_type > 0) {  // some thing to do
  	  high_r=r;
      // here is the big if
      if (evaluate(r)) {
 	    do_rule(r);	 // then do the assertion
  	    xnumfired++;
	    if (!get_fired(r)) {  // on transition 
	      if (logging&1)  	 	// do logging
	  	    do_log(r+1,LOG_EVENT);
	    }
	    set_fired(r,1);
	  }
	  else {
	    if (get_fired(r)) {  // on transition
	      if (logging&1)  	 	// do logging off dropped event
	  	    do_log(r+NUMEVENTS+1,LOG_EVENT);
		} 
	    set_fired(r,0);
	  }
    }
  } 
  if (!(force& FORCETONE)) 
     set_tone(newtone);
  if (xnumfired)	 		// setup blue flash time - faster = more events
    blueflash= 100 - (xnumfired * 5);
  else
    blueflash= 200;
  numfired =xnumfired;
  // handle clearing functions
  if (x10_fun) {
    lx_x10_fun=x10_fun;
  }
  if (disp_rule > high_r) disp_rule=0;
  if  ((sec & 0x1)==0) disp_rule++;
  x10_fun=0;
  logging &=0x7F;	   // reset the logging now flag
  
}


/* EVALUATE - returns current state of input */
unsigned char evaluate(unsigned char r)
{
unsigned char eval;
unsigned char xtype;
unsigned char val1,val2;
int tx,t1,t2;

    eval=0;
	xtype = ev.in1_type;
	val1=ev.in1_val[0];
	val2=ev.in1_val[1];
    if  (xtype == INPUTTIME) {  // time check
	  // convert to minutes and check by the minte count
	  t1= ((val1 & 3) * 15) + ((val1 >> 2) * 60);
	  t2= ((val2 & 3) * 15) + ((val2 >> 2)*60);
	  tx = (hour * 60) + min;
	  if (t1 > t2) { // bizzar probably wants it past midnight
	    if (tx <= t1) {
		  if (t2 <= tx) {  // within range
		 	eval=1; 
		  }
		}
	  }
	  else {
	    if (tx >= t1) {
		  if (t2 >= tx) {  // within range
		 	eval=1;
	   	  } 
		}
	  } 
	} 
    else if (xtype ==INPUTLIGHT) {
	  q = lightavgsum >> 3;
	  if (logging&0x80)  	 	// do logging for current ligt  
	  	 do_log(q,LOG_VALUE_LIGHT);
      if ((val1 <= q) && (val2 >= q))
	      eval=1;
	}
	else if (xtype ==INPUTDOW) {  // date/dasy evaluate
	  if (val1 == dow)
	    eval=1; 
	  else {
	    if ((val1 == 8) && (dow > 1) && (dow < 7))  // weekday
		  eval = 1;
	    if ((val1 == 9) && ((dow ==1) || (dow == 7)))  // weekend!
		  eval = 1;
	    if ((val1 > 9) && ((val1-9) == day))  // day of month
		  eval = 1;
		   
	  }
	}
	else if (xtype ==INPUTTEMP) {  // temp 1 (int) evaluate
	  q = conv_temp(curtemp);
      if (logging&0x80) 	 	// do logging for current temp  
	  	 do_log(curtemp,LOG_VALUE_TEMP);
	  if ((val1 <= q) && (val2 >= q))
	    eval=1; 
	}
    else if (xtype ==LOG_VALUE_TEMPB) {  // temp 2 (ext) evaluate
	  q = conv_temp(curtemp2);
      if (logging&0x80) 	 	// do logging for current temp  
	  	 do_log(curtemp2,LOG_VALUE+2);
	  if ((val1 <= q) && (val2 >= q))
	    eval=1; 
	}
    else if (xtype ==INPUTX10) {
      eval = get_fired(r);		   // default - last known value
	  if (x10_fun) {
	    if (main_x10_hc==x10_hcode) {
  	      // new X10 fucntion to look at
   	      if (val1 == x10_addr) {
            if (logging&0x80)  	 	// do logging for value  
	      	 do_log(x10_fun+ (x10_addr<<4),LOG_VALUE_X10);
		  // address and house code match do something?
		   if (val2 == x10_fun) {  // ok we have an match
      	    set_fired(r,0);  // always transition
	        eval=1; 
		    disp_rule=r;
		   }
		   else  // our address matches, but different function - return false
	        eval=0; 
		  }
		}
	  }
	}
    else if (xtype ==INPUTIR) {
      eval = get_fired(r);	   // default - zero
  	  if (newIR > 0) {
	    if (val1 == newIR) {
  		  set_fired(r,0);  // always transition
	      eval=1;
		  disp_rule=r;
		}
		else
	      eval=0;
	  } 
	}
    else if (xtype ==INPUTFLAG) {
       if ((flag >= val1) && (flag <=val2))
	      eval=1;
	  else
	     eval=0;
	}
    else if (xtype ==INPUTDIGITAL) {
  	  ADSCR=0x1D;  // change the A-D channel to a non input pin
  	  val1--;  // make zero based
      q = (PTB & (1 << (val1))?1:0);
	  if (q == (val2))
	    eval=1; 
	}
    else if (xtype ==INPUTANALOG) {
 	 ADCLK=0x10;
   	 ADSCR = 0x20;   // continious conversions, select A/D
  	 while (!(ADSCR&0x80));
     q = ADR;
 	 if (logging&0x80)  	 	// do logging for current analog value  
	  	 do_log(q,LOG_VALUE_ANLG);
     if ((q >= val1) && (q <=val2))
	    eval=1; 
	}
    else if (xtype == INPUTDLYS) {	// start timer
    	if ((delaytimer>=val1) && (delaytimer<=val2))
		  eval=1;
	}
    else if (xtype == INPUTDLYM) {	// start timer
    	if ((delaytimerm>=val1) && (delaytimerm<=val2))
		  eval=1;
	}
	else
	  eval=0;
	 
	// ok check to see if we have logic types
	val1=ev.in_logic;
	if (val1 > 0) {
	   if (val1 <= NUMEVENTS) { // AND
	     val2 = val1-1;
	     if (get_fired(val2) && eval)
		   eval = 1;
		 else
		   eval = 0;
	   } 
	   else {  // must be OR
	     val2 = val1-(NUMEVENTS+1);
	     if (get_fired(val2) || eval)
		   eval = 1;
		 else
		   eval = 0;
	   } 	
	}
	// handle background status display - every other second
    if ((lcdmenumode==LCDMENUMODE_RUNSTATUS) && ((sec & 0x1)==0)) {
	  // we are displaying running status 
	  if (disp_rule == r) {  
	    // and this is the rule we want to display 
		backdisplay=xtype;  // all other displays is the Xtype
	  }
    }
	return(eval);

}

/* DO_RULE - OK Assert */
void do_rule(unsigned char r)
{

    if  (ev.out1_type == OUTPUTHEALTH) {
     if (!(force&FORCEHEALTH))
	   healthcode = ev.out1_val[0];
	}
    else if (ev.out1_type ==OUTPUTRELAY) {
	  if (!(force&FORCERELAY)) {
	    if (relaymode == 0) 
    	    set_relay(ev.out1_val[0]-1,ev.out1_val[1]);
	    if ((relaymode == 1) && !get_fired(r))  // once on first time thorugh 
    	    set_relay(ev.out1_val[0]-1,ev.out1_val[1]);
	    if ((relaymode == 2) && (sec & 1))  // Flash 
    	    set_relay(ev.out1_val[0]-1,ev.out1_val[1]);
	  }
	}
    else if ((ev.out1_type ==OUTPUTOUTPUT)) {
	  if (!(force&FORCEOUTPUT)) {
  	      digitalout=2 << (ev.out1_val[0]);
	      if (ev.out1_val[1]) {  // ON
	        kp_io_reg2 |= digitalout;
	      }
	      else { 		   // OFF 
	        kp_io_reg2 &= ~digitalout;
		  }
	  }
	}
    else if ((ev.out1_type ==OUTPUTEXP)) {
	  if (!(force&FORCEOUTPUT)) {
  	      digitalout=0x80 >> (ev.out1_val[0]-1);
	      if (ev.out1_val[1]) {  // ON
	        exp_io_reg |= digitalout;
	      }
	      else { 		   // OFF 
	        exp_io_reg &= ~digitalout;
		  }
	  }
	}
    else if (ev.out1_type ==OUTPUTTONE) {
	  newtone = ev.out1_val[0];
	}
    else if (ev.out1_type ==OUTPUTFLAG) {
	  if (!get_fired(r)) {
	    if (ev.out1_val[0] == 10) {
		  if (flag < 9) flag++;
		}
	    else if (ev.out1_val[0] == 11) {
		  if (flag > 0) flag--;
		 }
	    else flag = ev.out1_val[0];
	  }
	}
    else  if (ev.out1_type ==OUTPUTX10) {  // send  X10
  	  if (!get_fired(r)) { // if x10 first time thorugh
	   q = ev.out1_val[0];
       send_X10(graycode[q&0xF] << 1,q >> 4);
	   send_X10((ev.out1_val[1]*2)+1,q >> 4);
	  }
	}
    else  if (ev.out1_type ==OUTPUTDLYS) { // start delay secs
	 if (!get_fired(r)) 
       delaytimer=ev.out1_val[0];
	}
    else  if (ev.out1_type ==OUTPUTDLYM) {  // start selay mins
	 if (!get_fired(r)) // delay
       delaytimerm=ev.out1_val[0];
	}

}