/*   spi.c  - SPI (Serial Peroph Interface)  handler

  handles:
    SPI (bit shift) out
    SPI (bit shift) in

The 68HC08 processor does most the work, just load the data and wait for a complete flag. 
Once the data is shifted out a latch bit (also called the select bit) is toggled to latch the 
shifted data out to a register.

	WARNING: These loops DO NOT timeout - they do check for kbhit for aborts

	MAJOR ROUTINES:
unsigned char spi_read(void)  -   read data from SPI return 1 byte 
void spi_write(unsigned char dat,unsigned char select_bit ) - write a byte of data to SPI, select bit to strobe  passed in
void spi_write2(unsigned char dat,unsigned char dat2,unsigned char select_bit )	- writes a 2 byte buffer of data to SPI, select bit to strobe  passed in
	
    Version      Author   Date           Comments
     1.00           hg.   7/4/02  Initial Development
*/

#define SPILIB 1

/** PROTOTYPES **/
void spi_write(unsigned char dat,unsigned char select_bit );
void spi_write2(unsigned char dat,unsigned char dat2,unsigned char select_bit );
unsigned char spi_read(void);


/* SPI_READ - read data from SPI return 1 byte */
unsigned char spi_read(void)
{
unsigned char dat;

  dat = SPSCR;	  // clear the read status
  dat = SPDR;
  while ((!(SPSCR & 0x08)) && !kbhit());   /* wait for write transfer to end */
  SPDR=0x69;          /* send some junk data */
  while ((!(SPSCR & 0x80)) && !kbhit());   /* wait for read transfer to end */
  return(SPDR);       /* return the read data */
}

/* SPI_WRITE -  write a byte of data to SPI, select bit to strobe  passed in */
void spi_write(unsigned char dat,unsigned char select_bit )
{
  while ((!(SPSCR & 0x08)) && !kbhit());   /* wait for last transfer to end */
  SPDR=dat;          /* send some data */
  while ((!(SPSCR & 0x08)) && !kbhit());   /* wait for last transfer to end */
  PTC|=select_bit;   /* strobe it out */
  PTC&=~select_bit;
  PTC|=select_bit;
  return;
}

/* SPI_WRITE2 - writes a 2 byte buffer of data to SPI, select bit to strobe  passed in*/
/* SPI_WRITE2 is used for the keypad_io registers -Called on the GSI-2 for the KB_IO stuff */
void spi_write2(unsigned char dat,unsigned char dat2,unsigned char select_bit )
{
  while ((!(SPSCR & 0x08)) && !kbhit());   /* wait for last transfer to end */
  SPDR=dat2;          /* send some data */
  while ((!(SPSCR & 0x08)) && !kbhit());   /* wait for last transfer to end */
  SPDR=dat;          /* send some more data */
  while ((!(SPSCR & 0x08)) && !kbhit());   /* wait for last transfer to end */
  PTC|=select_bit;   /* strobe it out */
  PTC&=~select_bit;
  PTC|=select_bit;
  return;
}
 
 

