/*   temp.c  - Temp Probe (DS1821 and DS1822) handling

  handles:
    Temp Probe Chip (DS1821) And also DS1822's
    one wire interface -

Notes: The DS1821 is a thermostat with limited resolution
	   The DS1822 is cheaper and has better resolution, now supported
	   
	   During INIT the temp sensors are searched and serial numbers recorded
	   Only two temp sensors are supported because of the way the search routine is written. More could be supported
	   with changes to the search routine.
	   Once the Serial numbers are recorded, they are used for every access 
	
	WARNING: timing sensitive - turn off ints
	
	Only DS1822 currently supported, DS1821 conditional compile

	This will handle up to 2 DS1822 probes or one 1821 probe
	
		MAJOR ROUTINES:
unsigned char temp_Presense() - check to see if at least one temp sensor alive  - 1 = ok, 0 = no temp sense 
unsigned char temp_Init_1822(void) - Temp init - scan for sensors RETURNS: Number of sensors found
void Read_Temp(void)- reads current temp and puts is curtemp varable
int conv_temp(int intemp) - Convert into correct units for 1822 
int conv_temp_dp(int intemp) -- Convert into correct (1/10t digit) units for 1822  Retuns a int of the temp to right of DP un current units

    Version      Author   Date           Comments
     1.00           hg.   12/27/00  Initial Development
*/
/** PROTOTYPES **/
void temp_One(void);
void temp_Zero(void);
unsigned char temp_Presense(void);
void temp_Write(unsigned char cmd);
char temp_bit_Read(void);
int conv_temp(int intemp);
int conv_temp_dp(int intemp);
unsigned char temp_Init_1822(void);

#define TEMPLIB 1
#define DS1822	1
//#define DS1821	1 		  // Uncomment or the old 1821 thermostat
//#define DEBUGTEMP 1		  // uncomment for debugging output

// DELAYS !!! from scopping signals, timing sensative, uses caldelay

// HARD TINMING Defines - done on GSI-2A with scope
// these are called by teh CALDELAY (vary timing and CPU clock sensitive)
#define OW_PRE_ONE 18   // was 36
#define OW_PRE_ZERO 100	   // 620 us
#define OW_PRE_DELAY 100   // 620 us min
#define OW_WRITE_ONE 20 
#define OW_READ_DELAY 10   // aprox 60 us
#define OW_READ2_DELAY 20
#define OW_WRITE_ZERO 15   // 60 - 120 us 
#define OW_WRITE_ZERO_DELAY 4   // 15 us min
#define OW_WRITE_ONE_DELAY 10   // 60 us min



 /* Temp Sensor OK flag - if presense fails - this incated something wrong and don't  keep trying*/
// AND 1822 count (99 = 1821 instead)
unsigned char ds1822_num;	/* number of ds1822 (0 = no sensors, 1-2 = DS1822, 99 = DS1821 */
unsigned char temp_SN1[8];	   // Seruial number fo the sensors - filled in in init
unsigned char temp_SN2[8];
int curtemp,curtemp2;			   /* current temp holders */

int mintemp,maxtemp;		   /* statistics */
int mintemp2,maxtemp2;		   /* statistics for external sensor */


/* TEMP_ONE - write a one out */
void temp_One()
{
  PTA&=~PA_ONE_WIRE;   // needs > 1us bus less than 15us
  PTA&=~PA_ONE_WIRE;
  PTA&=~PA_ONE_WIRE;
  PTA|=PA_ONE_WIRE;
  caldelay(OW_WRITE_ONE_DELAY);
}

/* TEMP_ZERO - write a zero out */
void temp_Zero()
{
  PTA&=~PA_ONE_WIRE;
  caldelay(OW_WRITE_ZERO);
  PTA|=PA_ONE_WIRE;
  caldelay(OW_WRITE_ZERO_DELAY);
}


/* TEMP_PRESENSE - check to see if at least one temp sensor alive  - 1 = ok, 0 = no temp sense */
unsigned char temp_Presense()
{
int q,i;
#define TEMPP_TIMEOUT 30  /* temp probe presense timeout */

      PTA|=PA_ONE_WIRE;
      DDRA|=PA_ONE_WIRE;  /* turn on the output data bit */
      PTA&=~PA_ONE_WIRE;      // send reset pulse for 500us min
      caldelay(OW_PRE_ZERO);
      PTA|=PA_ONE_WIRE;
      DDRA&=~PA_ONE_WIRE;  /* turn off the output data bit */
      /* wait for presense */
      q=i =0;
	  // should be high awhile
      while ((PTA&PA_ONE_WIRE) && (i < TEMPP_TIMEOUT)) i++;
	  // then go low for 60 - 240 us
      while (((~(PTA)&PA_ONE_WIRE)) && (q < TEMPP_TIMEOUT)) q++;
      if ((i==TEMPP_TIMEOUT) || (q == TEMPP_TIMEOUT)) {
#ifdef DEBUGTEMP
        printf("Temp Probe Timeout!\n\r");
#endif	
        return(0);
      }
      else {
	  return(1);
	}
}

/* temp_Write - send command byte  */
void temp_Write(unsigned char cmd)
{
unsigned char i,q;

    DDRA|=PA_ONE_WIRE;  /* turn on the output data bit */
    q=1;
    for (i=0;i<8;i++) {
      if (q & cmd) temp_One();  /* set data bit */
      else temp_Zero();			 /* reset data bit */
      q = q << 1;
    }
    PTA |=PA_ONE_WIRE;
    caldelay(OW_WRITE_ONE);  /* ok wait a little */
}

/* temp_bit_Read - read 1 bit */
char temp_bit_Read()
{
      /* strobe it */
      DDRA|=PA_ONE_WIRE;  /* turn on the output data bit */
      PTA&=~PA_ONE_WIRE;
      PTA&=~PA_ONE_WIRE;
      PTA&=~PA_ONE_WIRE;  // > 1 us
      DDRA&=~PA_ONE_WIRE;  /* turn off the output data bit - floats back high */
	  DDRA&=~PA_ONE_WIRE;
	  DDRA&=~PA_ONE_WIRE;
      return(PTA & PA_ONE_WIRE);  /* set data bit if line set just now*/
}

/* temp_REad - read data byte  */
unsigned char temp_Read()
{
unsigned char i,q,inbyte;


  inbyte = 0;
  q=1;
  for (i=0;i<8;i++) {
      if (temp_bit_Read()) inbyte |= q;  /* set data bit if line set just now*/
      q = q << 1;
      caldelay(OW_READ_DELAY);
  }
  PTA |=PA_ONE_WIRE;
  return(inbyte);
}

#ifdef DS1821	

/* Temp init - start cont. Conversions */
void temp_Init_1821(void)
{


  /* reset */
  if (temp_Presense()) {
    temp_Write(0x0C);  /* write status */
    temp_Write(0xD0);  /* Enable cont. conversion */
  }
  if (temp_Presense()) {
    temp_Write(0xEE);   /* start conversions */
  }
}


/* Read_Temp - reads current temp and puts is curtemp varable */
void Read_Temp_1821(void)
{
int tempx;

      asm("sei");
      if (temp_Presense() ==0) {
        asm("cli");
        return;  /* set reset pulse and detection */
      }
      temp_Write(0xAA);
      tempx = temp_Read();
      curtemp = tempx*16;
      if (curtemp < mintemp)
	  mintemp=curtemp;
      if (curtemp > maxtemp)
	  maxtemp=curtemp;
      asm("cli");
}
#endif




/* temp_Search - search for temp sensors */
/* Returns: 1 = OK, 0= none found */
unsigned char temp_Search(unsigned char first)
{
unsigned char i,q,z;

      asm("sei");
      if (temp_Presense() ==0)  {
        asm("cli");
 	    return 0 ;  /* set reset pulse and detection */
      }
      temp_Write(0xF0);  /* get the ROM read command */
      for (i=0; i<8; i++) {
        z=0;
        for (q=0; q<8; q++) {
	      if (!temp_bit_Read()) {
            caldelay(OW_READ_DELAY);
  	        if (!temp_bit_Read()) {
	        /* 0, 0 = devices have 0 and 1 */
              caldelay(10);
              PTA |=PA_ONE_WIRE;
              DDRA|=PA_ONE_WIRE;  /* turn on the output data bit */
              if (first==0)  /* select first one */
	            temp_Zero();
 	          else {	     /* naw, select second one this time */
	            temp_One();
    	        z|=1<<q;
	          }
	    }
	    else {
	      /* 0, 1 =  both  device has zero */
          caldelay(OW_READ2_DELAY);
          PTA |=PA_ONE_WIRE;
          DDRA|=PA_ONE_WIRE;  /* turn on the output data bit */
	      temp_Zero();
	    }
	  } 
	  else {
	    /* 1 - all devices have a 1 */
            caldelay(OW_READ_DELAY);
  	        temp_bit_Read();
            caldelay(OW_READ2_DELAY);
            PTA |=PA_ONE_WIRE;
 	        z|=1<<q;
            DDRA|=PA_ONE_WIRE;  /* turn on the output data bit */
	        temp_One();
	  }
          PTA |=PA_ONE_WIRE;
          caldelay(OW_READ2_DELAY);  /* ok wait a little */
	}
	if (first==0)
  	  temp_SN1[i]=z;
    else
      temp_SN2[i]=z;
#ifdef DEBUGTEMP
    printf("%X ", z);
#endif
  }
  asm("cli");
   return(1);
}

/* Temp init - scan for sensors */
/* RETURNS: Number of sensors found */
unsigned char temp_Init_1822(void)
{
unsigned char i,q;
#ifdef DEBUGTEMP
      printf("\n\rTemp Sensor 1 SN:");
#endif	  
      if (temp_Search(0) == 0) {
	    return(0);
	  }
      if (temp_SN1[0] == 255)   /* bad sn must be an old 1821 */
	    return(99);

#ifdef DEBUGTEMP
      printf("\n\rTemp Sensor 2 SN:");
#endif	  
      temp_Search(1);
      /* check to see if they are different */

      q = 0;
      for (i=0;i<8;i++) {
        if (temp_SN1[i] != temp_SN2[i]) q=1;
      }
      /* ok start conversion */
      asm("sei");
      if (temp_Presense() ==0)  {
        asm("cli");
	    return(0);  /* set reset pulse and detection */
      }
      temp_Write(0xCC);  /* skip the ROM  command - go to all sensors */
      temp_Write(0x4E);  /* write scratch pad */
      temp_Write(0x7F);  /* temp higg limit */
      temp_Write(0x0);  /* temp low */
      temp_Write(0x7F);  /* config - resilution */
      if (temp_Presense() ==0)  {
        asm("cli");
	    return(0);  /* set reset pulse and detection */
      }
      temp_Write(0xCC);  /* skip the ROM  command - go to all sensors */
      temp_Write(0x44);  /* convert T */
      asm("cli");
      if (q) /* different ones found */
	    return(2);
      else 
        return(1);
}



/* Read_Temp_22 - reads current temp from 1822 and puts is curtemp varable */
void Read_Temp_1822(void)
{
int tempx;
unsigned int i,q;

      asm("sei");		 // very timing sensitive
      if (temp_Presense() ==0)  {
        asm("cli");
	    return;  /* set reset pulse and detection */
      }
      /* select the device */
      temp_Write(0x55);  /* get the ROM read command */
      for (i=0; i<8; i++) {
	    temp_Write(temp_SN1[i]);
      }
      temp_Write(0xBE);  /* send scratch */
#ifdef DEBUGTEMP
	  // DEBUG to show everything comming back from temp (9 bytes)
      for (i=0; i<9; i++) {
	    q = temp_Read();
	    printf("..%X..",q);
		if (i==0) tempx = q;
		if (i==1) tempx = tempx + (q<<8);
      }
       printf("->%d\n\r",tempx);
#else	
	  tempx = temp_Read();
      q = temp_Read();
      tempx = tempx + (q<<8);
#endif	  
      // set high and low
      curtemp = tempx;
      if (curtemp < mintemp)   mintemp=curtemp;
      if (curtemp > maxtemp)   maxtemp=curtemp;
      if (ds1822_num == 2) {
	
        if (temp_Presense() ==0)  {
          asm("cli");
  	       return;  /* set reset pulse and detection */
        }
        /* select the device */
        temp_Write(0x55);  /* get the ROM read command */
        for (i=0; i<8; i++) {
	      temp_Write(temp_SN2[i]);
        }
        temp_Write(0xBE);  /* send scratch */
        tempx = temp_Read();
        q = temp_Read();
        tempx = tempx + (q<<8);
        curtemp2 = tempx;
          if (curtemp2 < mintemp2)   mintemp2=curtemp2;
          if (curtemp2 > maxtemp2)   maxtemp2=curtemp2;
      }
      /* ok start new conversion */
      if (temp_Presense() ==0)  {
        asm("cli");
   	    return;  /* set reset pulse and detection */
      }
      temp_Write(0xCC);  /* skip the ROM  command - go to all sensors */
      temp_Write(0x44);  /* convert T */
      asm("cli");  // reenable ints
}

/* Read_Temp - reads current temp and puts is curtemp varable */
void Read_Temp(void)
{
  if (ds1822_num < 99)
    Read_Temp_1822();	/* number of ds1822 */
#ifdef DS1821	
  else if (ds1822_num == 99)
    Read_Temp_1821();	/* number of ds1822 */
#endif	
}

/* CONV_TEMP - Convert into correct units for 1822 */
/* Retuns a int of the temp un current units */
int conv_temp(int intemp)
{
int tx;
 
 if (tempunits == 0)  return(intemp/16);
  else return(((intemp*9)/80)+32);
}

/* CONV_TEMP_DP - Convert into correct (1/10th digit) units for 1822 */
// Retuns a int of the temp to right of DP un current units
int conv_temp_dp(int intemp)
{
int tx;
 
 if (tempunits == 0)  return(((intemp%16) * 9)/15);
 else return((((intemp*9)/8)+320) % 10);
}


