/*   x10.c  - X10 (TW-523 and Powerlinc) handler

  handles:
    X10 send and recv. 
    included in main to add X10 functionality
	
Notes: use 	wait_trans to determine if the X10 is zero crossing sending pulses ok
	   use background_x10 in background processing loop to receive
	   to send a command you must call send_X10 twice, once with address then once with x10 function 
	
	MAJOR ROUTINES:
void wait_trans(void) - wait for transition of zero line
void background_x10(void) - Handle X10 Interface
unsigned int get_X10(void) - get a x10 command 
void send_X10(unsigned char x,unsigned char hc)- send an X10 command OR Address 


    Version      Author   Date           Comments
     1.00           hg.   7/11/00  Initial Development
*/
#define X10LIB 1
//#define DEBUGX10 1 	  // debugging output

/** PROTOTYPES **/
unsigned int get_X10(void);
void send_x10_0(void);
void send_x10_1(void);
void send_X10(unsigned char x,unsigned char hc);
void wait_trans(void);
void background_x10(void);
void txpulse(void);

const char hcode[16] = {
//12,0,2,10,14,6,0,8,0,5,3,11,15,0,1,4
12,4,2,10,14,6,0,8,13,5,3,11,15,7,1,9
};
#ifdef DEBUGX10
const char lbl[32][7] =
{ "Inv   ","A-Off ","Ad5   ","A-On  ","Ad 3  ","On    ","Ad11 ","OFF   ","Ad 15 ","Dim   ","Ad 7","Brght","Ad 1",
"A-Lof","Ad 9","ext","Ad 14","hail","Ad 6","HailA","Ad 4","Pre","Ad 12","Pre",
"Ad 16","ext","Add 8","st on","Ad 2","st Off","Ad 10","St req"
};
#endif

// used for house code look up
const char graycode[16] = {
6,14,2,10,1,9,5,13,7,15,3,11,0,8,4,12};


// X10 Command Defines
#define X10_CMD_ALLOFF 0
#define X10_CMD_ALLON  1
#define X10_CMD_ON     2
#define X10_CMD_OFF    3
#define X10_CMD_DIM    4
#define X10_CMD_BRIGHT 5

/* X10 varbles Stuff */  
unsigned int cx1,cx2;
unsigned char X10ok;    /* OK flag - set on startup if X10 interface working */

/* X10 Receive */
unsigned char x10_hcode;   // Last House Code received
unsigned char x10_addr;	   // Last Address
unsigned char x10_fun;	   // Last Function (clear to zero)

/* X10 Send */
unsigned char x10_sendaddr;	  // send address

/* BACKGROUND_X10 - Handle X10 Interface  */
/* called from background (keypress) */
void background_x10(void)
{
unsigned char qval;
unsigned int hc,val;

  if (!(PTC & PC_X10_IN)) {
    i = get_X10();
    if ((i > 0) & ((i & 0xF000) != 0xE000)) {
	#ifdef DEBUGX10
        printf("X10cmd-> Invalid- %X\n\r",i);
	#endif
	  return;
	}
    else if (i > 0) {
	  hc = (i >> 8) & 0xF;
      q=i&0x1f;  
	#ifdef DEBUGX10
      printf("X10rcv-> H:%d F:%s\n\r",hc,lbl[q]);
	#endif                                               	
	  if (q & 1) {
	    val = q >> 1;
        x10_fun = val;
		#ifdef DEBUGX10
		qval = 'A';
		qval += x10_hcode;
        printf("X10cmd-> Address %c %d, Funct %d... %d %d\n\r",qval,x10_addr,val,main_x10_addr,main_x10_hc);	
		#endif                                               	
	  }
	  else {
	    val = q >> 1;
	    x10_addr = hcode[val]+1;
  	    x10_hcode = hcode[hc];
		x10_fun=0;
	  }

// check for main house code and addess to set to
	     if ((main_x10_hc < 16) && (x10_hcode==main_x10_hc)) {

		   if (x10_addr == (main_x10_addr+1)) {  // Our houscode and address
		     
  		     //if (x10_fun == X10_CMD_ON) {	   // buzz command
			 //  set_tone(1);
		  	 //  printf("Buzz!!\n\r");
		     // }
		     if (x10_fun == X10_CMD_OFF)	{   // set time sync command
	  		   sclk_Write(SCLK_HOUR,to_BCD(3));
			   sclk_Write(SCLK_MIN,to_BCD(0));
			   sclk_Write(SCLK_SEC,to_BCD(0));
			   min=sec=0;
			   hour=3;
	           if (logging)  	 	// do logging - log startup
	             do_log(0xFFFD,LOG_GENERAL);		// FFFD = Time SYNC
 		      #ifdef DEBUGX10
		        printf("Time Sync!!\n\r");
			  #endif
  		     }
		     if (x10_fun == X10_CMD_DIM)	{   // reset health forces
  			    healthcode=0;
				force = 0;
  		     }
		     if ((x10_fun == X10_CMD_BRIGHT) &&	(healthcode < 4)){   // set blue command
			    healthcode++;
				force |= FORCEHEALTH;
		     }
		     x10_fun=0;   // set to 0 to indicate handled
		   }
	   }
	  
    }
   }
}
/* GET_X10 - get a x10 command */
unsigned int get_X10(void)
{
unsigned int x,q;
unsigned char i,p;

  asm("sei"); 	  // Disable ints. time sensitive

  cx1=cx2=0;
  x = 0x80;
  p = 0x40;
  while (PTC & PC_X10_IN);
  for (i=0;i<3;i++) {
    wait_trans();
    if (!(PTC & PC_X10_IN))  
        x += p;
    p = p >> 1;
	
  }
  for (i=0;i<4;i++) {
    wait_trans();
    if (!(PTC & PC_X10_IN))  {
        x += p;
    }
    wait_trans();
    if (!(PTC & PC_X10_IN))  {
        cx1 += p;
    }
    p = p >> 1;
  }
  q=0;
  p = 0x10;
  for (i=0;i<5;i++) {
    wait_trans();
    if (!(PTC & PC_X10_IN))  {
        q += p;
    }
    wait_trans();
    if (!(PTC & PC_X10_IN))  {
        cx2 += p;
    }
    p = p >> 1;
  }
  asm("cli");
  if (((~x) & 0xF) != cx1) {
#ifdef DEBUGX10
    printf("Error in check code %x!=%x\n\r",((~x) & 0xF),cx1);
#endif
    return(0);	   
  }
  if (((~q) & 0x1F) != cx2) {
#ifdef DEBUGX10
    printf("Error in check code2 %x!=%x\n\r",((~q) & 0x1F),cx2);
#endif
    return(0);	   
  }
  return((x << 8) + q);  
}

/* WAIT_TRANS - wait for transition of zero line */
void wait_trans(void)
{
int i;
  delay_v=31;
  if ((PTC & PC_X10_ZERO))
    while ((PTC & PC_X10_ZERO) && (delay_v > 0));
  else  
    while (!(PTC & PC_X10_ZERO) && (delay_v > 0));
  caldelay(15);
}

/* SEND_X10 - send an X10 command OR Address */
void send_X10(unsigned char x,unsigned char hc)
{
unsigned char i,q;
unsigned char qq;
unsigned char hcx;

  if (X10ok != TRUE) return;  // safty check
  /* outer loop - number of times to send sequence */
  for (qq=0;qq <2 ; qq++) {
    /* send start header */
    while (!(PTC & PC_X10_ZERO)); /* start high */
    txpulse();
    while ((PTC & PC_X10_ZERO));
    txpulse();
    while (!(PTC & PC_X10_ZERO));
    txpulse();
    while ((PTC & PC_X10_ZERO));
    PTC &= ~PC_X10_OUT;
	  /* send house code */
    q = 8;
	hcx = graycode[hc];
    for (i=0;i<4;i++) {
     if (hcx & q)  {
       send_x10_1();
     }
     else {
       send_x10_0();
     }
	 q = q >> 1;
    }
    /* now send data 5 bits */
    q = 0x10;
    for (i=0;i<5;i++) {
    	if (x & q)  {
        send_x10_1();
      }
      else {
        send_x10_0();
      }
	  	q = q >> 1;
    }
    /* gotta wait cycles now */
    wait_trans();
    wait_trans();
    wait_trans();
    wait_trans();
    wait_trans();
    wait_trans();
  }
}

#define TXPULSE 250

/* TXPULSE */
/* Timed puble - send out 1 ms pulse */
void txpulse(void)
{
unsigned char i;

    PTC |= PC_X10_OUT;
    caldelay(TXPULSE);
    PTC &= ~PC_X10_OUT;
  
}
/* SEND_X10_1 */
void send_x10_1(void)
{

    while (!(PTC & PC_X10_ZERO));
    txpulse();
    while ((PTC & PC_X10_ZERO));
}

/* SEND_X10_0 */
void send_x10_0(void)
{

    while (!(PTC & PC_X10_ZERO));
    PTC &= ~PC_X10_OUT;
    while ((PTC & PC_X10_ZERO));
    txpulse();
}

